# Copyright (c) Meta Platforms, Inc. and affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

from dataclasses import dataclass


@dataclass
class LogGroup:
    log_group_name: str



#!/usr/bin/env python3
# Copyright (c) Meta Platforms, Inc. and affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict
import abc

from fbpcp.entity.certificate_request import CertificateRequest


class CertificateService(abc.ABC):
    @abc.abstractmethod
    def __init__(self, cert_request: CertificateRequest, exe_path: str) -> None:
        pass

    @abc.abstractmethod
    def generate_certificate(
        self,
    ) -> str:
        pass



# Copyright (c) Meta Platforms, Inc. and affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

from dataclasses import dataclass
from typing import Any, Dict


RoleId = str
RoleName = str
PolicyName = str
PolicyContents = Dict[str, Any]


@dataclass
class IAMRole:
    role_id: RoleId
    attached_policy_contents: Dict[PolicyName, PolicyContents]



#!/usr/bin/env python3
# Copyright (c) Meta Platforms, Inc. and affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict
import logging
import re
from typing import List

from fbpcp.entity.policy_settings_config import PolicySettingsConfig
from fbpcp.entity.policy_statement import PolicyStatement
from fbpcp.service.policy_validation import PolicyValidationService


class AWSPolicyValidationService(PolicyValidationService):
    def __init__(self) -> None:
        self.logger: logging.Logger = logging.getLogger(__name__)

    def _principal_match(self, principal_settings: str, principal: str) -> bool:
        """Returns whether the principal matches the principal_settings.
        Args:
            principal_settings: Principal settings in the PolicySettingsConfig.
                This can either be a normal string,
                or in format "re(<principal regex>)" to indicate regex matching.
            principal: The principal string.
        """
        match = re.fullmatch(r"re\((.*?)\)", principal_settings)
        if match:
            principal_regex = match.group(1)
            return bool(re.fullmatch(principal_regex, principal))
        return principal_settings == principal

    def _policy_exists_in_statements(
        self,
        statements: List[PolicyStatement],
        effect: str,
        principal: str,
        actions: List[str],
        resources: List[str],
    ) -> bool:
        for statement in statements:
            contain_principal = any(
                self._principal_match(principal, stmt_principal)
                for stmt_principal in statement.principals
            )
            if (
                contain_principal
                and effect == statement.effect
                and set(actions).issubset(statement.actions)
                and set(resources).issubset(statement.resources)
            ):
                return True
        return False

    def is_bucket_policy_statements_valid(
        self,
        bucket: str,
        bucket_statements: List[PolicyStatement],
        policy_settings: List[PolicySettingsConfig],
    ) -> bool:
        for rule in policy_settings:
            if rule.exist != self._policy_exists_in_statements(
                bucket_statements,
                rule.effect,
                rule.principal,
                rule.actions,
                rule.resources,
            ):
                self.logger.error(
                    "The policy of bucket %s does not satisfy the following policy settings: %s.",
                    bucket,
                    str(rule),
                )
                return False
        return True



# Copyright (c) Meta Platforms, Inc. and affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

from typing import Any, Dict, Optional

import boto3
import botocore
from fbpcp.gateway.aws import AWSGateway
from pce.entity.log_group_aws import LogGroup


class LogsGateway(AWSGateway):
    def __init__(
        self,
        region: str,
        access_key_id: Optional[str] = None,
        access_key_data: Optional[str] = None,
        config: Optional[Dict[str, Any]] = None,
    ) -> None:
        super().__init__(region, access_key_id, access_key_data, config)

        self.client: botocore.client.BaseClient = boto3.client(
            "logs", region_name=self.region, **self.config
        )

    def describe_log_group(self, log_group_name: str) -> Optional[LogGroup]:
        response = self.client.describe_log_groups(logGroupNamePrefix=log_group_name)
        """
        Only 1 log group name will be expected in this case
        Though this API returns up to 50 matches and a nextToken is required for pagination,  the first match will be the exact log_group_name
        """
        for group in response["logGroups"]:
            if group["logGroupName"] == log_group_name:
                log_group = LogGroup(log_group_name=group["logGroupName"])
                return log_group



#!/usr/bin/env python3
# Copyright (c) Meta Platforms, Inc. and affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

from typing import Any, Dict, Optional

from onedocker.entity.metadata import PackageMetadata
from onedocker.gateway.dynamodb import DynamoDBGateway
from onedocker.mapper.aws import map_dynamodbitem_to_packagemetadata


class MetadataService:
    def __init__(
        self,
        region: str,
        table_name: str,
        key_name: str,
        access_key_id: Optional[str] = None,
        access_key_data: Optional[str] = None,
        config: Optional[Dict[str, Any]] = None,
    ) -> None:
        self.table_name = table_name
        self.key_name = key_name
        self.dynamodb_gateway = DynamoDBGateway(
            region, access_key_id, access_key_data, config
        )

    def _build_key(self, package_name: str, version: str) -> str:
        return f"{package_name}#{version}"

    def get_medadata(self, package_name: str, version: str) -> PackageMetadata:
        dynamodb_item = self.dynamodb_gateway.get_item(
            table_name=self.table_name,
            key_name=self.key_name,
            key_value=self._build_key(package_name=package_name, version=version),
        )

        return map_dynamodbitem_to_packagemetadata(dynamodb_item=dynamodb_item)

    def put_metadata(self, metadata: PackageMetadata) -> None:
        md_dict = metadata.to_dict()
        md_dict.update(
            {self.key_name: self._build_key(metadata.package_name, metadata.version)}
        )
        self.dynamodb_gateway.put_item(table_name=self.table_name, item=md_dict)



#!/usr/bin/env python3
# Copyright (c) Meta Platforms, Inc. and affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

from base64 import b64decode, b64encode
from typing import Any, Dict, List, Optional

import boto3
from botocore.client import BaseClient
from fbpcp.decorator.error_handler import error_handler
from fbpcp.gateway.aws import AWSGateway


class KMSGateway(AWSGateway):
    def __init__(
        self,
        region: str,
        access_key_id: Optional[str] = None,
        access_key_data: Optional[str] = None,
        config: Optional[Dict[str, Any]] = None,
    ) -> None:
        super().__init__(region, access_key_id, access_key_data, config)
        self.client: BaseClient = boto3.client(
            "kms", region_name=self.region, **self.config
        )

    @error_handler
    def sign(
        self,
        key_id: str,
        message: str,
        message_type: str,
        grant_tokens: List[str],
        signing_algorithm: str,
    ) -> str:
        response = self.client.sign(
            KeyId=key_id,
            Message=message.encode(),
            MessageType=message_type,
            GrantTokens=grant_tokens,
            SigningAlgorithm=signing_algorithm,
        )
        signature = b64encode(response["Signature"]).decode()
        return signature

    @error_handler
    def verify(
        self,
        key_id: str,
        message: str,
        message_type: str,
        signature: str,
        signing_algorithm: str,
        grant_tokens: List[str],
    ) -> bool:
        b64_signature = b64decode(signature.encode())
        response = self.client.verify(
            KeyId=key_id,
            Message=message.encode(),
            MessageType=message_type,
            Signature=b64_signature,
            SigningAlgorithm=signing_algorithm,
            GrantTokens=grant_tokens,
        )
        return response["SignatureValid"]



# Copyright (c) Meta Platforms, Inc. and affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict
from dataclasses import dataclass
from enum import Enum
from typing import List


class Effect(str, Enum):
    ALLOW = "Allow"
    DENY = "Deny"


@dataclass
class PolicySettingsConfig:
    """This is a config struct used by Repository Monitor to set constraints for bucket policy.
    Args:
        exist: Wether the specified permission exists in the bucket's policy.
        effect: The specified effect.
        principal: The specified principal. Use normal string or "re(<principal regex>)" for regex matching.
        actions: List of specified actions.
        resources: List of resources.
    """

    exist: bool
    effect: Effect
    principal: str
    actions: List[str]
    resources: List[str]



#!/usr/bin/env python3
# Copyright (c) Meta Platforms, Inc. and affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

from fbpcp.error.pcp import InvalidParameterError, PcpError, ThrottlingError
from kubernetes.client.exceptions import (
    ApiException,
    ApiTypeError,
    ApiValueError,
    OpenApiException,
)

# reference: https://github.com/kubernetes-client/python/blob/d3de7a85a63fa6bec6518d1cc75dc5e9458b9bbc/kubernetes/client/exceptions.py
def map_k8s_error(error: OpenApiException) -> PcpError:
    message = str(error)
    if isinstance(error, (ApiValueError, ApiTypeError)):
        return InvalidParameterError(message)
    elif isinstance(error, ApiException):
        code = error.status
        if code == 429:
            return ThrottlingError(message)
        if code == 400 or code == 404:
            return InvalidParameterError(message)

    return PcpError(message)



# Copyright (c) Meta Platforms, Inc. and affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

from typing import Any, Dict, Optional

import boto3
from fbpcp.gateway.aws import AWSGateway


class STSGateway(AWSGateway):
    def __init__(
        self,
        region: str,
        access_key_id: Optional[str] = None,
        access_key_data: Optional[str] = None,
        config: Optional[Dict[str, Any]] = None,
    ) -> None:
        super().__init__(region, access_key_id, access_key_data, config)

        # pyre-ignore
        self.client = boto3.client("sts", region_name=self.region, **self.config)

    def get_caller_arn(
        self,
    ) -> str:
        response = self.client.get_caller_identity()

        return response["Arn"]



#!/usr/bin/env python3
# Copyright (c) Meta Platforms, Inc. and affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict
from concurrent.futures import ThreadPoolExecutor
from typing import Any, Dict, Final, Iterator, List, Optional, Tuple

import boto3
from botocore.client import BaseClient
from fbpcp.decorator.error_handler import error_handler
from fbpcp.decorator.metrics import duration_time, error_counter, request_counter
from fbpcp.entity.cluster_instance import Cluster
from fbpcp.entity.container_definition import ContainerDefinition
from fbpcp.entity.container_instance import ContainerInstance
from fbpcp.error.pcp import PcpError
from fbpcp.gateway.aws import AWSGateway
from fbpcp.mapper.aws import (
    map_ecstask_to_containerinstance,
    map_ecstaskdefinition_to_containerdefinition,
    map_esccluster_to_clusterinstance,
    map_gb_to_mb,
    map_vcpu_to_unit,
)
from fbpcp.metrics.emitter import MetricsEmitter
from fbpcp.metrics.getter import MetricsGetter

METRICS_RUN_TASK_COUNT = "aws.ecs.run_task.count"
METRICS_RUN_TASK_ERROR_COUNT = "aws.ecs.run_task.error.count"
METRICS_RUN_TASK_DURATION = "aws.ecs.run_task.duration"


class ECSGateway(AWSGateway, MetricsGetter):
    def __init__(
        self,
        region: str,
        access_key_id: Optional[str] = None,
        access_key_data: Optional[str] = None,
        config: Optional[Dict[str, Any]] = None,
        metrics: Optional[MetricsEmitter] = None,
        session_token: Optional[str] = None,
    ) -> None:
        super().__init__(region, access_key_id, access_key_data, config, session_token)

        self.client: BaseClient = self.create_ecs_client()
        self.metrics: Final[Optional[MetricsEmitter]] = metrics

    def has_metrics(self) -> bool:
        return self.metrics is not None

    def get_metrics(self) -> MetricsEmitter:
        if not self.metrics:
            raise PcpError("ECSGateway doesn't have metrics emitter")

        return self.metrics

    # TODO: Create an interface to create a client per environment
    def create_ecs_client(
        self,
    ) -> BaseClient:
        return boto3.client("ecs", region_name=self.region, **self.config)

    @error_counter(METRICS_RUN_TASK_ERROR_COUNT)
    @request_counter(METRICS_RUN_TASK_COUNT)
    @duration_time(METRICS_RUN_TASK_DURATION)
    @error_handler
    def run_task(
        self,
        task_definition: str,
        container: str,
        cmd: str,
        cluster: str,
        subnets: List[str],
        env_vars: Optional[Dict[str, str]] = None,
        cpu: Optional[int] = None,
        memory: Optional[int] = None,
    ) -> ContainerInstance:
        environment = []
        if env_vars:
            environment = [
                {"name": env_name, "value": env_value}
                for env_name, env_value in env_vars.items()
            ]
        container_override: Dict[str, Any] = {
            "name": container,
            "command": [cmd],
            "environment": environment,
        }
        if cpu and memory:
            cpu, memory = map_vcpu_to_unit(cpu), map_gb_to_mb(memory)
            container_override.update(
                {
                    "cpu": cpu,
                    "memory": memory,
                }
            )
            override = {
                "containerOverrides": [container_override],
                "cpu": str(cpu),
                "memory": str(memory),
            }
        else:
            override = {
                "containerOverrides": [container_override],
            }
        response = self.client.run_task(
            taskDefinition=task_definition,
            cluster=cluster,
            networkConfiguration={
                "awsvpcConfiguration": {
                    "subnets": subnets,
                    "assignPublicIp": "ENABLED",
                }
            },
            overrides=override,
        )

        if not response["tasks"]:
            # common failures: https://docs.aws.amazon.com/AmazonECS/latest/developerguide/api_failures_messages.html
            failure = response["failures"][0]
            self.logger.error(f"ECSGateway failed to create a task. Failure: {failure}")
            raise PcpError(f"ECS failure: reason: {failure['reason']}")

        return map_ecstask_to_containerinstance(response["tasks"][0])

    @error_handler
    def describe_tasks(
        self, cluster: str, tasks: List[str]
    ) -> List[Optional[ContainerInstance]]:
        response = self.client.describe_tasks(
            cluster=cluster, tasks=tasks
        )  # not necessarily in order of `tasks`

        arn_to_instance: Dict[str, Optional[ContainerInstance]] = {}
        for resp_task_dict in response["tasks"]:
            arn_to_instance[
                resp_task_dict["taskArn"]
            ] = map_ecstask_to_containerinstance(resp_task_dict)

        for failure in response["failures"]:
            self.logger.warning(
                f"ECSGateway failed to describe a task {failure['arn']}, reason: {failure['reason']}"
            )

        return [arn_to_instance.get(arn, None) for arn in tasks]

    @error_handler
    def describe_task(self, cluster: str, task: str) -> Optional[ContainerInstance]:
        return self.describe_tasks(cluster, [task])[0]

    @error_handler
    def list_tasks(
        self,
        cluster: str,
        nextToken: Optional[str] = None,
    ) -> Tuple[List[str], str]:
        kwargs = {"cluster": cluster}
        if nextToken:
            kwargs["nextToken"] = nextToken

        response = self.client.list_tasks(**kwargs)
        return response.get("taskArns", None), response.get("nextToken", None)

    @error_handler
    def iterate_list_tasks(
        self,
        cluster: str,
    ) -> Iterator[str]:
        list_tasks = self.list_tasks(cluster)
        if list_tasks[0]:
            yield from list_tasks[0]

        while list_tasks[1]:
            list_tasks = self.list_tasks(cluster, list_tasks[1])
            if list_tasks[0]:
                yield from list_tasks[0]

    @error_handler
    def stop_task(self, cluster: str, task_id: str) -> None:
        self.client.stop_task(
            cluster=cluster,
            task=task_id,
        )

    @error_handler
    def describe_clusters(
        self,
        clusters: Optional[List[str]] = None,
        tags: Optional[Dict[str, str]] = None,
    ) -> List[Cluster]:
        if not clusters:
            clusters = self.list_clusters()
        response = self.client.describe_clusters(clusters=clusters, include=["TAGS"])
        cluster_instances = [
            map_esccluster_to_clusterinstance(cluster)
            for cluster in response["clusters"]
        ]
        if tags:
            return list(
                filter(
                    lambda cluster_instance: tags.items()
                    <= cluster_instance.tags.items(),
                    cluster_instances,
                )
            )
        return cluster_instances

    @error_handler
    def describe_cluster(self, cluster: str) -> Cluster:
        return self.describe_clusters(clusters=[cluster])[0]

    @error_handler
    def list_clusters(self) -> List[str]:
        return self.client.list_clusters()["clusterArns"]

    @error_handler
    def describe_task_definition(self, task_defination: str) -> ContainerDefinition:
        return self._describe_task_definition_core(self.client, task_defination)

    def _describe_task_definition_core(
        self,
        client: BaseClient,
        task_defination: str,
    ) -> ContainerDefinition:
        response = client.describe_task_definition(
            taskDefinition=task_defination, include=["TAGS"]
        )
        return map_ecstaskdefinition_to_containerdefinition(
            response["taskDefinition"], response["tags"]
        )

    @error_handler
    def list_task_definitions(self, limit: int = 1000) -> List[str]:
        task_definitions = []
        next_token = ""

        while len(task_definitions) < limit:
            task_definition_response = self.client.list_task_definitions(
                nextToken=next_token,
            )
            task_definitions.extend(task_definition_response["taskDefinitionArns"])
            next_token = task_definition_response.get("nextToken")
            if not next_token:
                break

        if limit and len(task_definitions) > limit:
            task_definitions = task_definitions[:limit]
        return task_definitions

    @error_handler
    def describe_task_definitions(
        self,
        task_definitions: Optional[List[str]] = None,
        tags: Optional[Dict[str, str]] = None,
    ) -> List[ContainerDefinition]:
        if not task_definitions:
            task_definitions = self.list_task_definitions()
        container_definitions = []
        for arn in task_definitions:
            container_definition = self.describe_task_definition(arn)
            if tags is None or tags.items() <= container_definition.tags.items():
                container_definitions.append(container_definition)

        return container_definitions

    @error_handler
    def describe_task_definitions_in_parallel(
        self,
        task_definitions: Optional[List[str]] = None,
        tags: Optional[Dict[str, str]] = None,
        max_workers: int = 8,
    ) -> List[ContainerDefinition]:
        if not task_definitions:
            task_definitions = self.list_task_definitions()
        container_definitions = []
        with ThreadPoolExecutor(max_workers=max_workers) as executor:
            input_arguments = [
                (
                    self.create_ecs_client(),
                    definition,
                )
                for definition in task_definitions
            ]
            results = executor.map(
                lambda args: self._describe_task_definition_core(*args),
                input_arguments,
            )
            for container_definition in results:
                if tags is None or tags.items() <= container_definition.tags.items():
                    container_definitions.append(container_definition)

        return container_definitions



#!/usr/bin/env python3
# Copyright (c) Meta Platforms, Inc. and affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

# patternlint-disable f-string-may-be-missing-leading-f

from enum import Enum

from pce.validator.message_templates.pce_standard_constants import (
    CONTAINER_CPU,
    CONTAINER_IMAGE,
    CONTAINER_MEMORY,
    FIREWALL_RULE_FINAL_PORT,
    FIREWALL_RULE_INITIAL_PORT,
    IGW_ROUTE_DESTINATION_CIDR_BLOCK,
)

# PCE networking validation error messages
class NetworkingErrorTemplate(Enum):
    # VPC
    VPC_NON_PRIVATE_CIDR = "The CIDR of the vpc {vpc_cidr} is not a private range."

    # Firewall
    FIREWALL_CIDR_NOT_OVERLAPS_VPC = "VPC peering for VPC {peer_target_id} doesn't have an inbound rule to allow traffic to {vpc_id}:{vpc_cidr}."
    FIREWALL_CIDR_CANT_CONTAIN_EXPECTED_RANGE = f"Ingress cidr {{fr_vpc_id}}:{{fri_cidr}}:{{fri_from_port}}-{{fri_to_port}} does not contain all required ports from {FIREWALL_RULE_INITIAL_PORT} to {FIREWALL_RULE_FINAL_PORT}"
    FIREWALL_INVALID_RULESETS = "Invalid firewall rulesets: {error_reasons}."
    FIREWALL_PEER_ROUTE_NOT_SET = "No peer route found."
    FIREWALL_RULES_NOT_FOUND = "No firewall rules found tagged with {pce_id}."

    # Subnets
    SUBNETS_NOT_ALL_AZ_USED = "Subnets are not using all availability zones from {region}, currently using {azs}."

    # Route table
    ROUTE_TABLE_IGW_MISSING = "Internet Gateway route missing in route table."
    ROUTE_TABLE_IGW_INACTIVE = "Internet Gateway route is not Active."
    ROUTE_TABLE_VPC_PEERING_MISSING = (
        "No Active route for VPC Peering Connection found in the route table."
    )

    # VPC peering
    VPC_PEERING_NO_VPC_PEERING = "No VPC peering set."
    VPC_PEERING_NO_VPC = "VPC not found."
    VPC_PEERING_NO_VPC_CIDR = "No VPC CIDR set."
    VPC_PEERING_NO_ROUTE_TABLE = "No route table set."
    # Unacceptable states include VpcPeeringState.REJECTED and VpcPeeringState.NOT_READY
    # The latter maps any other state not explicitly listed in the VpcPeeringState Enum
    VPC_PEERING_UNACCEPTABLE_STATE = "VPC Peering is in unacceptable state: {status}."

    # Network unknown error
    NETWORKING_UNKNOWN = "PCE Networking unknown error"


# PCE Compute validation error messages
class ComputeErrorTemplate(Enum):
    # Container cluster
    CLUSTER_DEFINITION_NOT_SET = "No container definition."
    CLUSTER_DEFINITION_WRONG_VALUES = (
        "Container values incorrectly set: {error_reasons}"
    )
    CLUSTER_DEFINITION_WRONG_VALUE = (
        "{resource_name} value '{value}' is incorrect, expected '{expected_value}'."
    )
    ROLE_WRONG_POLICY = "None of the policies ({policy_names}) attached to the role {role_name} conform to standard."
    ROLE_POLICIES_NOT_FOUND = "Policies not attached to {role_names}."

    # Compute unknown error
    UNKNOWN = "PCE Compute Unknown error"


# PCE Networking solution hint
class NetworkingErrorSolutionHintTemplate(Enum):
    # VPC
    ROUTE_TABLE_VPC_PEERING_MISSING = (
        "Ensure that a route for VPC Peering Connection is present and Active in the route table. "
        "If present but not Active, check if the VPC Peering Connection request is pending "
        "acceptance by the owner of the Acceptor VPC."
    )

    # Firewall
    FIREWALL_CIDR_CANT_CONTAIN_EXPECTED_RANGE = (
        "Update the port range for {sec_group} to {from_port}-{to_port}."
    )
    FIREWALL_INVALID_RULESETS = "To remediate: {error_remediation}"

    # Subnets
    SUBNETS_NOT_ALL_AZ_USED = (
        "Set the subnets so that all availability zones are used, {azs} are missing."
    )

    # Route table
    ROUTE_TABLE_IGW_MISSING = (
        f"Ensure that a route for destination {IGW_ROUTE_DESTINATION_CIDR_BLOCK} "
        "to an Internet Gateway target is added to the route table."
    )
    ROUTE_TABLE_IGW_INACTIVE = (
        "Ensure that the Internet Gateway route in the routing table is in the Active state. "
        "Recreate the Internet Gateway resource if needed or reach out to AWS Support for troubleshooting."
    )
    # VPC Peering
    VPC_NON_PRIVATE_CIDR = "Set a private CIDR {default_vpc_cidr} for the vpc."


class ComputeErrorSolutionHintTemplate(Enum):
    # Compute Cluster
    CLUSTER_DEFINITION_WRONG_VALUES = f"Please set container values (cpu, memory, image) as ({CONTAINER_CPU},{CONTAINER_MEMORY},'{CONTAINER_IMAGE}')"
    ROLE_WRONG_POLICY = "Set the policy of {role_name} to {role_policy}."
    ROLE_POLICIES_NOT_FOUND = (
        "Make sure there are policies attached to {role_names} in the pce {pce_id}."
    )



#!/usr/bin/env python3
# Copyright (c) Meta Platforms, Inc. and affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

# patternlint-disable f-string-may-be-missing-leading-f
from enum import Enum


class ResourceNamePlural(Enum):
    VPC = "VPCs"
    ROUTE_TABLE = "Route Tables"
    VPC_PEERING = "VPC Peerings"
    CLUSTER = "ECS Clusters"
    CONTAINER_DEFINITION = "Container Definitions"



#!/usr/bin/env python3
# Copyright (c) Meta Platforms, Inc. and affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict
from dataclasses import dataclass
from typing import List


@dataclass
class PolicyStatement:
    """AWS Policy Statement: https://docs.aws.amazon.com/IAM/latest/UserGuide/reference_policies_elements_statement.html"""

    effect: str
    principals: List[str]
    actions: List[str]
    resources: List[str]
    sid: str = ""


@dataclass
class PublicAccessBlockConfig:
    """AWS PublicAccessBlockConfiguration: https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-s3-bucket-publicaccessblockconfiguration.html"""

    block_public_acls: bool
    ignore_public_acls: bool
    block_public_policy: bool
    restrict_public_buckets: bool



#!/usr/bin/env python3
# Copyright (c) Meta Platforms, Inc. and affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict


import re
from functools import reduce
from typing import Any, Dict, List, Optional, Tuple, Union

from fbpcp.error.pcp import InvalidParameterError


def convert_dict_to_list(
    target_dict: Dict[str, str], key: str, value: str
) -> List[Dict[str, List[str]]]:
    """
    Input examples:
        target_dict: {"k1": "v1", "k2": "v2"}
        key: "Name",
        value: "Values"
    Output examples:
        [
            {"Name": "k1", "Values": ["v1"]},
            {"Name": "k2", "Values": ["v2"]},
        ]
    """
    return reduce(
        lambda x, y: [*x, {key: y, value: [target_dict[y]]}], target_dict.keys(), []
    )


def convert_list_to_dict(
    target_list: List[Dict[str, str]], key: str, value: str
) -> Dict[str, str]:
    """
    Input examples:
        target_list:
        [
            {"Name": "k1", "Values": "v1"},
            {"Name": "k2", "Values": "v2"},
        ]
        key: "Name",
        value: "Value"
    Output examples: {"k1": "v1", "k2": "v2"}
    """
    if target_list is not None:
        return reduce(lambda x, y: {**x, **{y[key]: y[value]}}, target_list, {})
    else:
        return {}


# pyre-ignore
def convert_obj_to_list(obj: Any) -> List:
    return obj if isinstance(obj, list) else [obj]


def get_json_values(json_elem: Union[str, Dict[str, Any]]) -> List[str]:
    """
    Input examples: {"k1": ["v1", "v2"], "k2": "v3"}
    Output examples: ["v1", "v2", "v3"]
    """
    if isinstance(json_elem, str):
        return [json_elem]
    ret = []
    for val in json_elem.values():
        ret.extend(convert_obj_to_list(val))
    return ret


def prepare_tags(tags: Dict[str, str]) -> Dict[str, str]:
    """
    Input examples: {"k1": "v1", "k2": "v2"}
    Output examples: {"tag:k1": "v1", "tag:k2", "v2"}
    """
    return reduce(lambda x, y: {**x, **{f"tag:{y}": tags[y]}}, tags.keys(), {})


def convert_vpc_tags_to_filter(
    tags: Optional[Dict[str, str]] = None, vpc_id: Optional[str] = None
) -> List[Dict[str, List[str]]]:
    """
    Input examples:
        tags [Optional]: {"k1": "v1", "k2": "v2"},
        vpc_id [Optional]: "vpc-id-ex"
    Output examples:
        [
            {"Name": "k1", "Values": ["v1"]},
            {"Name": "k2", "Values": ["v2"]},
            {"Name": "vpc-id", "Values": ["vpc-id-ex"]}
        ]
    """
    vpc_dict = {"vpc-id": vpc_id} if vpc_id else {}
    tags_dict = prepare_tags(tags) if tags else {}
    filter_dict = {**vpc_dict, **tags_dict}
    return convert_dict_to_list(filter_dict, "Name", "Values") if filter_dict else []


def get_container_definition_id(task_definition_id: str, container: str) -> str:
    # the reverse logic from https://fburl.com/code/ycdjih3q
    return f"{task_definition_id}#{container}"


def split_container_definition(container_definition: str) -> Tuple[str, str]:
    """
    container_definition = task_definition#container
    task_definition: ECS task definition, e.g. test-definition:1
    container: ECS cluster name, e.g. test-cluster
    example: test-definition:1#test-cluster
    """
    if not is_container_definition_valid(container_definition):
        raise InvalidParameterError(
            "Parameter container_definition must be in format <task definition name>:<revision>#<container name>"
        )
    s = container_definition.split("#")
    return (s[0], s[1])


def is_container_definition_valid(container_definition: str) -> bool:
    is_valid = bool(re.fullmatch(r"[\w\-/:]+?:\d+#[\w\-/]+?", container_definition))
    return is_valid



#!/usr/bin/env python3
# Copyright (c) Meta Platforms, Inc. and affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict
import abc
from typing import List

from fbpcp.entity.policy_settings_config import PolicySettingsConfig
from fbpcp.entity.policy_statement import PolicyStatement


class PolicyValidationService:
    @abc.abstractmethod
    def is_bucket_policy_statements_valid(
        self,
        bucket: str,
        bucket_statements: List[PolicyStatement],
        policy_settings: List[PolicySettingsConfig],
    ) -> bool:
        """Returns whether the bucket's policy is valid according to policy_settings"""
        pass



#!/usr/bin/env python3
# Copyright (c) Meta Platforms, Inc. and affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

# patternlint-disable f-string-may-be-missing-leading-f

from enum import Enum

from pce.validator.message_templates.pce_standard_constants import (
    FIREWALL_RULE_FINAL_PORT,
    FIREWALL_RULE_INITIAL_PORT,
)

# for scenarios should raise up warning messages are
# 1) the current PCE setup is not a blocker for running study, however not fully follow the PCE set up standard.
# 2) run time components are still in pending status


class NetworkingValidationWarningDescriptionTemplate(Enum):
    NETWORKING_VPC_PEERING_PEERING_NOT_READY = (
        "VPC Peering Connection request is pending acceptance."
    )
    NETWORKING_FIREWALL_CIDR_EXCEED_EXPECTED_RANGE = f"Ingress cidr {{fr_vpc_id}}:{{fri_cidr}}:{{fri_from_port}}-{{fri_to_port}} exceeds the expected port range {FIREWALL_RULE_INITIAL_PORT}-{FIREWALL_RULE_FINAL_PORT}"
    NETWORKING_FIREWALL_FLAGGED_RULESETS = (
        "These issues are not fatal but are worth noticing: {warning_reasons}"
    )


class ValidationWarningDescriptionTemplate(Enum):
    CLUSTER_DEFINITION_FLAGGED_VALUE = (
        "{resource_name} value '{value}' is not expected, should be '{expected_value}'."
    )
    CLUSTER_DEFINITION_FLAGGED_VALUES = (
        "Container has outlier values which are non-fatal: {warning_reasons}"
    )
    MORE_POLICIES_THAN_EXPECTED = (
        "Policies {policy_names} attached to {role_id} are not expected."
    )
    CLOUDWATCH_LOGS_NOT_CONFIGURED_IN_TASK_DEFINITION = (
        "CloudWatch logs are not configured in ECS task definition."
    )
    CLOUDWATCH_LOGS_NOT_FOUND = "CloudWatch logs are not found, please check if log group name {log_group_name_from_task} is correct or if this log group gets deleted "


class ValidationWarningSolutionHintTemplate(Enum):
    VPC_PEERING_PEERING_NOT_READY = "Please work with owner of Acceptor VPC (VPC ID: {accepter_vpc_id}) to accept peering request."
    MORE_POLICIES_THAN_EXPECTED = (
        "Consider removing additional policies to strengthen security."
    )



#!/usr/bin/env python3
# Copyright (c) Meta Platforms, Inc. and affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

from dataclasses import dataclass
from enum import Enum
from typing import Dict

from fbpcp.entity.cloud_provider import CloudProvider
from fbpcp.error.pcp import InvalidParameterError


class ContainerType(Enum):
    SMALL = "SMALL"
    MEDIUM = "MEDIUM"
    LARGE = "LARGE"


CONTAINER_TYPES: Dict[CloudProvider, Dict[ContainerType, Dict[str, int]]] = {
    CloudProvider.AWS: {
        ContainerType.SMALL: {"cpu": 1, "memory": 8},
        ContainerType.MEDIUM: {"cpu": 4, "memory": 30},
        ContainerType.LARGE: {"cpu": 16, "memory": 120},
    }
}


@dataclass
class ContainerTypeConfig:
    cpu: int  # Number of vCPU
    memory: int  # Memory in GB

    @classmethod
    def get_config(
        cls, cloud_provider: CloudProvider, container_type: ContainerType
    ) -> "ContainerTypeConfig":
        if cloud_provider != CloudProvider.AWS:
            raise InvalidParameterError(
                f"Cloud provider {cloud_provider} is not supported."
            )
        return cls(**CONTAINER_TYPES[cloud_provider][container_type])



#!/usr/bin/env python3
# Copyright (c) Meta Platforms, Inc. and affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

import ipaddress
import logging
from collections import defaultdict
from dataclasses import dataclass
from enum import Enum
from typing import Any, Dict, List, Optional, Tuple

import click
from fbpcp.entity.firewall_ruleset import FirewallRuleset
from fbpcp.entity.pce import PCE
from fbpcp.entity.route_table import Route, RouteState, RouteTargetType
from fbpcp.entity.vpc_instance import Vpc
from fbpcp.entity.vpc_peering import VpcPeeringState
from fbpcp.service.pce_aws import PCE_ID_KEY
from pce.entity.mpc_roles import MPCRoles
from pce.gateway.ec2 import EC2Gateway
from pce.gateway.ecs import ECSGateway
from pce.gateway.iam import IAMGateway
from pce.gateway.logs_aws import LogsGateway
from pce.validator.message_templates.error_message_templates import (
    ComputeErrorSolutionHintTemplate,
    ComputeErrorTemplate,
    NetworkingErrorSolutionHintTemplate,
    NetworkingErrorTemplate,
)
from pce.validator.message_templates.pce_standard_constants import (
    CONTAINER_CPU,
    CONTAINER_IMAGE,
    CONTAINER_MEMORY,
    DEFAULT_PARTNER_VPC_CIDR,
    DEFAULT_VPC_CIDR,
    FIREWALL_RULE_FINAL_PORT,
    FIREWALL_RULE_INITIAL_PORT,
    IGW_ROUTE_DESTINATION_CIDR_BLOCK,
    TASK_POLICY,
)
from pce.validator.message_templates.validator_step_names import ValidationStepNames
from pce.validator.message_templates.warning_message_templates import (
    NetworkingValidationWarningDescriptionTemplate,
    ValidationWarningDescriptionTemplate,
    ValidationWarningSolutionHintTemplate,
)


class ValidationResultCode(Enum):
    SUCCESS = "SUCCESS"
    ERROR = "ERROR"
    WARNING = "WARNING"


@dataclass
class ValidationResult:
    validation_result_code: ValidationResultCode = ValidationResultCode.ERROR
    validation_step_name: Optional[str] = None
    description: Optional[str] = None
    solution_hint: Optional[str] = None

    def __str__(self) -> str:
        return f"ValidationStepName.{self.validation_step_name}: {self.description}{f' {self.solution_hint}' if self.solution_hint else ''}"

    def __hash__(self) -> int:
        return hash(self.__str__())


class ClusterResourceType(Enum):
    CPU = "cpu"
    MEMORY = "memory"
    IMAGE = "image"


class ValidationSuite:
    def __init__(
        self,
        region: str,
        key_id: Optional[str] = None,
        key_data: Optional[str] = None,
        config: Optional[Dict[str, Any]] = None,
        role: Optional[MPCRoles] = None,
        ec2_gateway: Optional[EC2Gateway] = None,
        iam_gateway: Optional[IAMGateway] = None,
        ecs_gateway: Optional[ECSGateway] = None,
        logs_gateway: Optional[LogsGateway] = None,
    ) -> None:
        self.role: MPCRoles = role or MPCRoles.PARTNER
        self.ec2_gateway: EC2Gateway = ec2_gateway or EC2Gateway(
            region, key_id, key_data, config
        )
        self.iam_gateway: IAMGateway = iam_gateway or IAMGateway(
            region, key_id, key_data, config
        )
        self.ecs_gateway: ECSGateway = ecs_gateway or ECSGateway(
            region, key_id, key_data, config
        )

        self.logs_gateway: LogsGateway = logs_gateway or LogsGateway(
            region, key_id, key_data, config
        )

    def validate_vpc_cidr(self, pce: PCE) -> ValidationResult:
        default_vpc_cidr = (
            DEFAULT_PARTNER_VPC_CIDR
            if self.role == MPCRoles.PARTNER
            else DEFAULT_VPC_CIDR
        )
        vpc = pce.pce_network.vpc
        if not vpc:
            return ValidationResult(
                ValidationResultCode.ERROR,
                ValidationStepNames.VPC_CIDR.code_name,
                NetworkingErrorTemplate.VPC_PEERING_NO_VPC.value,
            )
        vpc_network = ipaddress.ip_network(vpc.cidr)
        valid_network = ipaddress.ip_network(default_vpc_cidr)
        is_valid = (
            vpc_network.is_private
            and (vpc_network[0] in valid_network)
            and (vpc_network[-1] in valid_network)
        )

        return (
            ValidationResult(
                ValidationResultCode.SUCCESS, ValidationStepNames.VPC_CIDR.code_name
            )
            if is_valid
            else ValidationResult(
                ValidationResultCode.ERROR,
                ValidationStepNames.VPC_CIDR.code_name,
                NetworkingErrorTemplate.VPC_NON_PRIVATE_CIDR.value.format(
                    vpc_cidr=vpc.vpc_id
                ),
                NetworkingErrorSolutionHintTemplate.VPC_NON_PRIVATE_CIDR.value.format(
                    default_vpc_cidr=DEFAULT_PARTNER_VPC_CIDR
                ),
            )
        )

    def validate_vpc_peering(self, pce: PCE) -> ValidationResult:
        if self.role == MPCRoles.PARTNER:
            vpc_peering = pce.pce_network.vpc_peering
        else:
            vpc = pce.pce_network.vpc
            vpc_peering_list = (
                self.ec2_gateway.describe_vpc_peering_connections_with_accepter_vpc_id(
                    vpc_id=vpc.vpc_id
                )
                if vpc
                else None
            )

            vpc_peering = vpc_peering_list[0] if vpc_peering_list else None

        if not vpc_peering:
            return ValidationResult(
                ValidationResultCode.ERROR,
                ValidationStepNames.VPC_PEERING.code_name,
                NetworkingErrorTemplate.VPC_PEERING_NO_VPC_PEERING.value,
            )

        state_results = defaultdict(
            lambda: ValidationResult(
                ValidationResultCode.ERROR,
                ValidationStepNames.VPC_PEERING.code_name,
                NetworkingErrorTemplate.VPC_PEERING_UNACCEPTABLE_STATE.value.format(
                    status=vpc_peering.status
                ),
            ),
            {
                VpcPeeringState.ACTIVE: ValidationResult(
                    ValidationResultCode.SUCCESS,
                    ValidationStepNames.VPC_PEERING.code_name,
                ),
                VpcPeeringState.PENDING_ACCEPTANCE: ValidationResult(
                    ValidationResultCode.WARNING,
                    ValidationStepNames.VPC_PEERING.code_name,
                    NetworkingValidationWarningDescriptionTemplate.NETWORKING_VPC_PEERING_PEERING_NOT_READY.value,
                    ValidationWarningSolutionHintTemplate.VPC_PEERING_PEERING_NOT_READY.value.format(
                        accepter_vpc_id=vpc_peering.accepter_vpc_id
                    ),
                ),
            },
        )

        return state_results[vpc_peering.status]

    def _check_inbound_peer_route_allowed(
        self,
        peer_routes: List[Route],
        firewall_rulesets: List[FirewallRuleset],
        vpc: Vpc,
    ) -> Tuple[List[str], List[str], List[str]]:
        """
        For each peer route check if there is an inbound rule for it. If there is, make sure the port range of the rule is within PCE standard range. The results are returned in 2 lists of messages, one for errors and one for warnings.
        """
        error_reasons = []
        error_remediation = []
        warning_reasons = []

        for peer_route in peer_routes:
            peer_cidr = peer_route.destination_cidr_block
            peer_rule_found = False
            for fr in firewall_rulesets:
                for fri in fr.ingress:
                    if not ipaddress.ip_network(fri.cidr).overlaps(
                        ipaddress.ip_network(peer_cidr)
                    ):
                        continue
                    peer_rule_found = True
                    if (
                        FIREWALL_RULE_INITIAL_PORT < fri.from_port
                        or FIREWALL_RULE_FINAL_PORT > fri.to_port
                    ):
                        error_reasons.append(
                            NetworkingErrorTemplate.FIREWALL_CIDR_CANT_CONTAIN_EXPECTED_RANGE.value.format(
                                fr_vpc_id=fr.vpc_id,
                                fri_cidr=fri.cidr,
                                fri_from_port=fri.from_port,
                                fri_to_port=fri.to_port,
                            ),
                        )
                        error_remediation.append(
                            NetworkingErrorSolutionHintTemplate.FIREWALL_CIDR_CANT_CONTAIN_EXPECTED_RANGE.value.format(
                                sec_group=fr.id,
                                from_port=FIREWALL_RULE_INITIAL_PORT,
                                to_port=FIREWALL_RULE_FINAL_PORT,
                            ),
                        )
                    elif (
                        FIREWALL_RULE_INITIAL_PORT > fri.from_port
                        or FIREWALL_RULE_FINAL_PORT < fri.to_port
                    ):
                        warning_reasons.append(
                            NetworkingValidationWarningDescriptionTemplate.NETWORKING_FIREWALL_CIDR_EXCEED_EXPECTED_RANGE.value.format(
                                fr_vpc_id=fr.vpc_id,
                                fri_cidr=fri.cidr,
                                fri_from_port=fri.from_port,
                                fri_to_port=fri.to_port,
                            )
                        )
            if not peer_rule_found:
                error_reasons.append(
                    NetworkingErrorTemplate.FIREWALL_CIDR_NOT_OVERLAPS_VPC.value.format(
                        peer_target_id=peer_route.route_target.route_target_id,
                        vpc_id=vpc.vpc_id,
                        vpc_cidr=vpc.cidr,
                    )
                )

        return error_reasons, error_remediation, warning_reasons

    def validate_firewall(self, pce: PCE) -> ValidationResult:
        """
        Check that inbound traffic from the peers is allowed i.e. there is a rule whose CIDR is overlapped by any firewall rule.
        """
        vpc = pce.pce_network.vpc
        if not vpc:
            return ValidationResult(
                ValidationResultCode.ERROR,
                ValidationStepNames.FIREWALL.code_name,
                NetworkingErrorTemplate.VPC_PEERING_NO_VPC.value,
            )
        vpc_cidr = vpc.cidr
        if not vpc_cidr:
            return ValidationResult(
                ValidationResultCode.ERROR,
                ValidationStepNames.FIREWALL.code_name,
                NetworkingErrorTemplate.VPC_PEERING_NO_VPC_CIDR.value,
            )
        # Don't bother in validating route table if there are no firewall rules
        firewall_rulesets = pce.pce_network.firewall_rulesets
        if not firewall_rulesets:
            return ValidationResult(
                ValidationResultCode.ERROR,
                ValidationStepNames.FIREWALL.code_name,
                NetworkingErrorTemplate.FIREWALL_RULES_NOT_FOUND.value.format(
                    pce_id=vpc.tags[PCE_ID_KEY]
                ),
            )
        route_table = pce.pce_network.route_table
        if not route_table:
            return ValidationResult(
                ValidationResultCode.ERROR,
                ValidationStepNames.FIREWALL.code_name,
                NetworkingErrorTemplate.VPC_PEERING_NO_ROUTE_TABLE.value,
            )
        peer_routes = [
            r
            for r in (route_table.routes if route_table else [])
            if r.route_target.route_target_type == RouteTargetType.VPC_PEERING
        ]
        if not peer_routes:
            return ValidationResult(
                ValidationResultCode.ERROR,
                ValidationStepNames.FIREWALL.code_name,
                NetworkingErrorTemplate.FIREWALL_PEER_ROUTE_NOT_SET.value,
            )

        (
            error_reasons,
            error_remediation,
            warning_reasons,
        ) = self._check_inbound_peer_route_allowed(peer_routes, firewall_rulesets, vpc)

        if error_reasons:
            return ValidationResult(
                ValidationResultCode.ERROR,
                ValidationStepNames.FIREWALL.code_name,
                NetworkingErrorTemplate.FIREWALL_INVALID_RULESETS.value.format(
                    error_reasons=";".join(error_reasons)
                ),
                NetworkingErrorSolutionHintTemplate.FIREWALL_INVALID_RULESETS.value.format(
                    error_remediation=";".join(error_remediation)
                ),
            )
        if warning_reasons:
            return ValidationResult(
                ValidationResultCode.WARNING,
                ValidationStepNames.FIREWALL.code_name,
                NetworkingValidationWarningDescriptionTemplate.NETWORKING_FIREWALL_FLAGGED_RULESETS.value.format(
                    warning_reasons=";".join(warning_reasons)
                ),
            )

        return ValidationResult(
            ValidationResultCode.SUCCESS, ValidationStepNames.FIREWALL.code_name
        )

    def validate_route_table(self, pce: PCE) -> ValidationResult:
        """
        Make sure routing table has all the required routes in active state
        This includes routes for a Peering Connection and an Internet Gateway
        """
        vpc = pce.pce_network.vpc
        if not vpc:
            return ValidationResult(
                ValidationResultCode.ERROR,
                ValidationStepNames.ROUTE_TABLE.code_name,
                NetworkingErrorTemplate.VPC_PEERING_NO_VPC.value,
            )
        route_table = pce.pce_network.route_table
        if not route_table:
            return ValidationResult(
                ValidationResultCode.ERROR,
                ValidationStepNames.ROUTE_TABLE.code_name,
                NetworkingErrorTemplate.VPC_PEERING_NO_ROUTE_TABLE.value,
            )

        is_vpc_peering_valid = any(
            r.state == RouteState.ACTIVE
            and r.route_target.route_target_type == RouteTargetType.VPC_PEERING
            for r in (route_table.routes if route_table else [])
        )

        if not is_vpc_peering_valid:
            return ValidationResult(
                ValidationResultCode.ERROR,
                ValidationStepNames.ROUTE_TABLE.code_name,
                NetworkingErrorTemplate.ROUTE_TABLE_VPC_PEERING_MISSING.value,
                NetworkingErrorSolutionHintTemplate.ROUTE_TABLE_VPC_PEERING_MISSING.value,
            )

        igw_route = None
        for route in route_table.routes:
            if (
                route.route_target.route_target_type == RouteTargetType.INTERNET
                and route.destination_cidr_block == IGW_ROUTE_DESTINATION_CIDR_BLOCK
            ):
                igw_route = route

        if not igw_route:
            return ValidationResult(
                ValidationResultCode.ERROR,
                ValidationStepNames.ROUTE_TABLE.code_name,
                NetworkingErrorTemplate.ROUTE_TABLE_IGW_MISSING.value,
                NetworkingErrorSolutionHintTemplate.ROUTE_TABLE_IGW_MISSING.value,
            )

        if igw_route.state != RouteState.ACTIVE:
            return ValidationResult(
                ValidationResultCode.ERROR,
                ValidationStepNames.ROUTE_TABLE.code_name,
                NetworkingErrorTemplate.ROUTE_TABLE_IGW_INACTIVE.value,
                NetworkingErrorSolutionHintTemplate.ROUTE_TABLE_IGW_INACTIVE.value,
            )

        return ValidationResult(
            ValidationResultCode.SUCCESS, ValidationStepNames.ROUTE_TABLE.code_name
        )

    def validate_subnets(self, pce: PCE) -> ValidationResult:
        """
        Check that all subnets make use of every AZ in the VPC region
        """
        region_azs = set(self.ec2_gateway.describe_availability_zones())
        used_azs = {s.availability_zone for s in pce.pce_network.subnets}
        # There can't be a case where AZs are used and are not among the region AZs
        is_valid = region_azs == used_azs
        return (
            ValidationResult(
                ValidationResultCode.SUCCESS, ValidationStepNames.SUBNETS.code_name
            )
            if is_valid
            else ValidationResult(
                ValidationResultCode.ERROR,
                ValidationStepNames.SUBNETS.code_name,
                NetworkingErrorTemplate.SUBNETS_NOT_ALL_AZ_USED.value.format(
                    region=pce.pce_network.region,
                    azs=",".join(sorted(used_azs)) if used_azs else "none",
                ),
                NetworkingErrorSolutionHintTemplate.SUBNETS_NOT_ALL_AZ_USED.value.format(
                    azs=",".join(sorted(region_azs - used_azs)),
                ),
            )
        )

    def validate_cluster_definition(self, pce: PCE) -> ValidationResult:
        """
        Check CPU, memory and image name values are according to the standard
        """
        c = pce.pce_compute.container_definition
        if not c:
            return ValidationResult(
                ValidationResultCode.ERROR,
                ValidationStepNames.CLUSTER_DEFINITION.code_name,
                ComputeErrorTemplate.CLUSTER_DEFINITION_NOT_SET.value,
            )
        message_by_level = defaultdict(list)
        for resource, expected_value, level in (
            (
                ClusterResourceType.CPU,
                CONTAINER_CPU,
                ValidationResultCode.ERROR,
            ),
            (
                ClusterResourceType.MEMORY,
                CONTAINER_MEMORY,
                ValidationResultCode.ERROR,
            ),
            (
                ClusterResourceType.IMAGE,
                CONTAINER_IMAGE,
                ValidationResultCode.WARNING,
            ),
        ):
            value = getattr(c, resource.value)
            if value != expected_value:
                message_template = (
                    ComputeErrorTemplate.CLUSTER_DEFINITION_WRONG_VALUE.value
                    if ValidationResultCode.ERROR == level
                    else ValidationWarningDescriptionTemplate.CLUSTER_DEFINITION_FLAGGED_VALUE.value
                )
                message = message_template.format(
                    resource_name=resource.value.title(),
                    value=value,
                    expected_value=expected_value,
                )
                message_by_level[level].append(message)

        if message_by_level[ValidationResultCode.ERROR]:
            return ValidationResult(
                ValidationResultCode.ERROR,
                ValidationStepNames.CLUSTER_DEFINITION.code_name,
                ComputeErrorTemplate.CLUSTER_DEFINITION_WRONG_VALUES.value.format(
                    error_reasons=",".join(message_by_level[ValidationResultCode.ERROR])
                ),
                ComputeErrorSolutionHintTemplate.CLUSTER_DEFINITION_WRONG_VALUES.value,
            )
        elif message_by_level[ValidationResultCode.WARNING]:
            return ValidationResult(
                ValidationResultCode.WARNING,
                ValidationStepNames.CLUSTER_DEFINITION.code_name,
                ValidationWarningDescriptionTemplate.CLUSTER_DEFINITION_FLAGGED_VALUES.value.format(
                    warning_reasons=",".join(
                        message_by_level[ValidationResultCode.WARNING]
                    )
                ),
            )
        return ValidationResult(
            ValidationResultCode.SUCCESS,
            ValidationStepNames.CLUSTER_DEFINITION.code_name,
        )

    def validate_network_and_compute(
        self,
        pce: PCE,
        skip_steps: Optional[List[ValidationStepNames]] = None,
        run_steps: Optional[List[ValidationStepNames]] = None,
    ) -> List[ValidationResult]:
        """
        Execute all existing validations returning warnings and errors encapsulated in `ValidationResult` objects
        """
        validation_steps: List[ValidationStepNames] = (
            run_steps if run_steps else list(ValidationStepNames)
        )
        if skip_steps:
            validation_steps = list(
                filter(lambda step: step not in skip_steps, validation_steps)
            )
            logging.info(
                f"Skipping the following validation steps: {', '.join([step.code_name for step in skip_steps])}"
            )

        logging.info(
            f"Running the following validation steps: {', '.join([step.code_name for step in validation_steps])}"
        )

        with click.progressbar(
            [
                (
                    self.__getattribute__(f"validate_{step.code_name}"),
                    step.formatted_name,
                )
                for step in validation_steps
            ],
            item_show_func=lambda i: str(i[1]) if i else "",
            label="Validating PCE...",
        ) as validation_functions:
            return [
                vr
                for vr in [
                    validation_function(pce)
                    for validation_function, _ in validation_functions
                ]
                if ValidationResultCode.SUCCESS != vr.validation_result_code
            ]

    @classmethod
    def summarize_errors(cls, validation_results: List[ValidationResult]) -> str:
        results_by_code = defaultdict(list)
        for result in validation_results:
            results_by_code[result.validation_result_code].append(result)
        summary = ""
        for code, results in results_by_code.items():
            for result in results:
                summary += f"{code}: {result}\n"
        return summary

    @classmethod
    def contains_error_result(cls, results: List[ValidationResult]) -> bool:
        return any(
            [
                result
                for result in results
                if result.validation_result_code == ValidationResultCode.ERROR
            ]
        )

    def validate_iam_roles(self, pce: PCE) -> ValidationResult:
        """
        Ensure that the container task execution role has the proper policy (`TASK_POLICY`) among those attached to it.
        """
        c = pce.pce_compute.container_definition
        if not c:
            return ValidationResult(
                ValidationResultCode.ERROR,
                ValidationStepNames.IAM_ROLES.code_name,
                ComputeErrorTemplate.CLUSTER_DEFINITION_NOT_SET.value,
            )

        policies = self.iam_gateway.get_policies_for_role(c.task_role_id)

        if not policies:
            pce_id = c.tags[PCE_ID_KEY]
            return ValidationResult(
                ValidationResultCode.ERROR,
                ValidationStepNames.IAM_ROLES.code_name,
                ComputeErrorTemplate.ROLE_POLICIES_NOT_FOUND.value.format(
                    role_names=c.task_role_id
                ),
                ComputeErrorSolutionHintTemplate.ROLE_POLICIES_NOT_FOUND.value.format(
                    role_names=c.task_role_id, pce_id=pce_id
                ),
            )

        policy_name_found = None
        for policy_name, policy_contents in policies.attached_policy_contents.items():
            if TASK_POLICY == policy_contents:
                policy_name_found = policy_name
                break

        if not policy_name_found:
            return ValidationResult(
                ValidationResultCode.ERROR,
                ValidationStepNames.IAM_ROLES.code_name,
                ComputeErrorTemplate.ROLE_WRONG_POLICY.value.format(
                    role_name=c.task_role_id,
                    policy_names=",".join(policies.attached_policy_contents.keys()),
                ),
                ComputeErrorSolutionHintTemplate.ROLE_WRONG_POLICY.value.format(
                    role_name=c.task_role_id,
                    role_policy=TASK_POLICY,
                ),
            )

        if len(policies.attached_policy_contents.values()) > 1:
            return ValidationResult(
                ValidationResultCode.WARNING,
                ValidationStepNames.IAM_ROLES.code_name,
                ValidationWarningDescriptionTemplate.MORE_POLICIES_THAN_EXPECTED.value.format(
                    policy_names=",".join(
                        policies.attached_policy_contents.keys() - {policy_name_found}
                    ),
                    role_id=c.task_role_id,
                ),
                ValidationWarningSolutionHintTemplate.MORE_POLICIES_THAN_EXPECTED.value,
            )
        return ValidationResult(
            ValidationResultCode.SUCCESS, ValidationStepNames.IAM_ROLES.code_name
        )

    def validate_log_group(self, pce: PCE) -> ValidationResult:
        c = pce.pce_compute.container_definition
        if not c:
            return ValidationResult(
                ValidationResultCode.ERROR,
                ValidationStepNames.LOG_GROUP.code_name,
                ComputeErrorTemplate.CLUSTER_DEFINITION_NOT_SET.value,
            )

        log_group_name_from_task = self.ecs_gateway.extract_log_group_name(c.id)
        if not log_group_name_from_task:
            return ValidationResult(
                ValidationResultCode.WARNING,
                ValidationStepNames.LOG_GROUP.code_name,
                ValidationWarningDescriptionTemplate.CLOUDWATCH_LOGS_NOT_CONFIGURED_IN_TASK_DEFINITION.value,
            )

        log_group = self.logs_gateway.describe_log_group(
            log_group_name=log_group_name_from_task
        )

        if log_group and log_group.log_group_name:
            return ValidationResult(
                ValidationResultCode.SUCCESS, ValidationStepNames.LOG_GROUP.code_name
            )

        return ValidationResult(
            ValidationResultCode.WARNING,
            ValidationStepNames.LOG_GROUP.code_name,
            ValidationWarningDescriptionTemplate.CLOUDWATCH_LOGS_NOT_FOUND.value.format(
                log_group_name_from_task=log_group_name_from_task
            ),
        )



#!/usr/bin/env python3
# Copyright (c) Meta Platforms, Inc. and affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict
from enum import Enum


class CloudProvider(Enum):
    AWS = "AWS"
    GCP = "GCP"
    AZURE = "AZURE"



#!/usr/bin/env python3
# Copyright (c) Meta Platforms, Inc. and affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

import asyncio
import logging
from typing import Dict, Final, List, Optional, Union

from fbpcp.decorator.metrics import duration_time, error_counter, request_counter
from fbpcp.entity.certificate_request import CertificateRequest
from fbpcp.entity.container_instance import ContainerInstance, ContainerInstanceStatus
from fbpcp.entity.container_type import ContainerType
from fbpcp.error.pcp import PcpError
from fbpcp.metrics.emitter import MetricsEmitter
from fbpcp.metrics.getter import MetricsGetter
from fbpcp.service.container import ContainerService
from fbpcp.util.arg_builder import build_cmd_args
from fbpcp.util.typing import checked_cast

ONEDOCKER_CMD_PREFIX = (
    # patternlint-disable-next-line f-string-may-be-missing-leading-f
    "python3.8 -m onedocker.script.runner {package_name} {runner_args}"
)

DEFAULT_BINARY_VERSION = "latest"

METRICS_CONTAINER_COUNT = "onedocker.container.count"
METRICS_START_CONTAINERS_COUNT = "onedocker.start_containers.count"
METRICS_START_CONTAINERS_ERROR_COUNT = "onedocker.start_containers.error.count"
METRICS_START_CONTAINERS_DURATION = "onedocker.start_containers.duration"

METRICS_FAILED_CONTAINERS_COUNT = "onedocker.failed.containers.count"
METRICS_REQUESTED_CONTAINERS_COUNT = "onedocker.requested.containers.count"


class OneDockerService(MetricsGetter):
    """OneDockerService is responsible for executing a package(binary) in a container on Cloud"""

    def __init__(
        self,
        container_svc: ContainerService,
        task_definition: Optional[str] = None,
        metrics: Optional[MetricsEmitter] = None,
    ) -> None:
        """Constructor of OneDockerService
        container_svc -- service to spawn container instances
        task_definition -- container definition to spawn container instances
        metrics -- metrics emitter to emit metrics
        """
        if container_svc is None:
            raise ValueError(f"Dependency is missing. container_svc={container_svc}, ")
        self.container_svc = container_svc
        self.task_definition = task_definition
        self.metrics: Final[Optional[MetricsEmitter]] = metrics
        self.logger: logging.Logger = logging.getLogger(__name__)

    def get_cluster(self) -> str:
        """Get the cluster of the container service

        Returns:
            The container service's cluster name
        """
        return self.container_svc.get_cluster()

    def has_metrics(self) -> bool:
        return self.metrics is not None

    def get_metrics(self) -> MetricsEmitter:
        if not self.metrics:
            raise PcpError("OneDocker doesn't have metrics emitter")

        return self.metrics

    def start_container(
        self,
        package_name: str,
        task_definition: Optional[str] = None,
        version: str = DEFAULT_BINARY_VERSION,
        cmd_args: str = "",
        env_vars: Optional[Union[Dict[str, str], List[Dict[str, str]]]] = None,
        timeout: Optional[int] = None,
        tag: Optional[str] = None,
        certificate_request: Optional[CertificateRequest] = None,
        container_type: Optional[ContainerType] = None,
    ) -> ContainerInstance:
        """
        This function statrts one container for running MPC games.

        Args:
            package_name:       Name of running package within docker image
            task_definition:    Task definition required by docker containers. If specified, overrides OneDockerService's task definition
                                when starting this container
            version:            The version of the MPC binary to run. This parameter defaults to the 'latest' binary version.
            cmd_args:           A string that is used to override the command in docker containers
            env_vars:           Environment variable overrides in docker containers. When given a single dictionary,
                                it will be applied to all container. When given a list of dictionraies, each will be assigned
                                to one container.
            timeout:            container timeout. If specified, docker container would be forced to stop
            tag:                Tag for docker containers
            certificate_request: An optional instance of CertificateRequest that contains the parameters required to create a TLS certificate

        """
        return self.start_containers(
            package_name=package_name,
            task_definition=task_definition,
            version=version,
            cmd_args_list=[cmd_args] if cmd_args else None,
            env_vars=env_vars,
            timeout=timeout,
            tag=tag,
            certificate_request=certificate_request,
            container_type=container_type,
        )[0]

    @error_counter(METRICS_START_CONTAINERS_ERROR_COUNT)
    @request_counter(METRICS_START_CONTAINERS_COUNT)
    @duration_time(METRICS_START_CONTAINERS_DURATION)
    def start_containers(
        self,
        package_name: str,
        task_definition: Optional[str] = None,
        version: str = DEFAULT_BINARY_VERSION,
        cmd_args_list: Optional[List[str]] = None,
        env_vars: Optional[Union[Dict[str, str], List[Dict[str, str]]]] = None,
        timeout: Optional[int] = None,
        tag: Optional[str] = None,
        certificate_request: Optional[CertificateRequest] = None,
        container_type: Optional[ContainerType] = None,
    ) -> List[ContainerInstance]:
        """Spin up cloud containers according to command arg list.

        Args:
            package_name:       Name of running package within docker image
            task_definition:    Task definition required by docker containers. If specified, overrides OneDockerService's task definition
                                when starting this container
            version:            The version of the MPC binary to run. This parameter defaults to the 'latest' binary version.
            cmd_args_list:      A list of command overrides in docker containers
            env_vars:           Environment variable overrides in docker containers. When given a single dictionary,
                                it will be applied to all container. When given a list of dictionraies, each will be assigned
                                to one container.
            timeout:            container timeout. If specified, docker container would be forced to stop
            tag:                Tag for docker containers
            certificate_request: An optional instance of CertificateRequest that contains the parameters required to create a TLS certificate

        Returns:
            A list of the containers that were successfuly started
        """
        if not cmd_args_list:
            raise ValueError("Command Argument List shouldn't be None or Empty")

        if type(env_vars) is list and len(env_vars) != len(cmd_args_list):
            raise ValueError(
                f"Length of env_vars {len(env_vars)} not equal to the length of cmd_args_list {len(cmd_args_list)}."
            )

        cmds = [
            self._get_cmd(package_name, version, cmd_args, timeout, certificate_request)
            for cmd_args in cmd_args_list
        ]
        container_type_log = f" of {container_type} type" if container_type else ""
        self.logger.info(
            f"Spinning up {len(cmds)} container instance[s]" + container_type_log
        )

        task_definition = task_definition or self.task_definition
        if not task_definition:
            raise ValueError(
                "task definition should be specified when spinning up containers"
            )
        containers = self.container_svc.create_instances(
            container_definition=task_definition,
            cmds=cmds,
            env_vars=env_vars,
            container_type=container_type,
        )
        if containers:
            self.logger.info(
                f"Spun up {len(containers)} ({containers[0].cpu}vCPU, {containers[0].memory}GB) containers"
            )

        if self.metrics:
            name = f"{METRICS_CONTAINER_COUNT}.{self.container_svc.get_region()}.{self.container_svc.get_cluster()}"
            name = f"{METRICS_CONTAINER_COUNT}.{tag}" if tag else name
            self.metrics.count(name, len(containers))
        return containers

    async def wait_for_pending_containers(
        self, container_ids: List[str]
    ) -> List[ContainerInstance]:
        tasks = [
            asyncio.create_task(self.wait_for_pending_container(container_id))
            for container_id in container_ids
        ]
        res = await asyncio.gather(*tasks)

        if self.metrics:
            failed_metrics = f"{METRICS_FAILED_CONTAINERS_COUNT}.{self.container_svc.get_region()}.{self.container_svc.get_cluster()}"
            requested_metrics = f"{METRICS_REQUESTED_CONTAINERS_COUNT}.{self.container_svc.get_region()}.{self.container_svc.get_cluster()}"
            self.metrics.count(requested_metrics, len(res))

            """ The following sum(condition(c) for c in list) will count the number of elements c that match a certain condition(c).
            This creates a generator expression returns True for each element that matches the condition and False otherwise.
            Since the True and False values are represented as integer values 1 and 0, by summing over the iterable, we will get the total number of matching elements.
            """
            failed_count = sum(
                not (container and container.status == ContainerInstanceStatus.STARTED)
                for container in res
            )
            self.metrics.count(
                failed_metrics,
                failed_count,
            )

        return [checked_cast(ContainerInstance, container) for container in res]

    async def wait_for_pending_container(
        self, container_id: str
    ) -> Optional[ContainerInstance]:
        updated_container = self.get_containers([container_id])[0]
        while (
            not updated_container
            or not updated_container.ip_address
            or updated_container.status is ContainerInstanceStatus.UNKNOWN
        ):
            await asyncio.sleep(1)
            updated_container = self.get_containers([container_id])[0]
            if updated_container is None:
                break
        return updated_container

    def stop_containers(self, containers: List[str]) -> List[Optional[PcpError]]:
        return self.container_svc.cancel_instances(containers)

    def get_containers(
        self, instance_ids: List[str]
    ) -> List[Optional[ContainerInstance]]:
        # TODO We will need long term discussion on the container capacity of onedocker
        """Get one or more container instances

        Args:
            instance_ids: a list of the container instances.

        Returns:
            A list of Optional, in the same order as the input ids. For example, if
            users pass 3 instance_ids and the second instance could not be found,
            then returned list should also have 3 elements, with the 2nd elements being None.
        """
        return self.container_svc.get_instances(instance_ids)

    def get_container(self, instance_id: str) -> Optional[ContainerInstance]:
        return self.container_svc.get_instance(instance_id)

    def _get_exe_name(self, package_name: str) -> str:
        return package_name.split("/")[1]

    def _get_cmd(
        self,
        package_name: str,
        version: str = DEFAULT_BINARY_VERSION,
        cmd_args: Optional[str] = None,
        timeout: Optional[int] = None,
        certificate_request: Optional[CertificateRequest] = None,
    ) -> str:
        args_dict = {"exe_args": cmd_args, "version": version, "timeout": timeout}
        if certificate_request:
            args_dict["cert_params"] = certificate_request.convert_to_cert_params()
        runner_args = build_cmd_args(**args_dict)
        return ONEDOCKER_CMD_PREFIX.format(
            package_name=package_name,
            runner_args=runner_args,
        ).strip()



#!/usr/bin/env python3
# Copyright (c) Meta Platforms, Inc. and affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

from typing import Dict, List, Optional

from fbpcp.error.pcp import PcpError
from fbpcp.service.storage import StorageService
from onedocker.entity.measurement import MeasurementType
from onedocker.entity.metadata import PackageMetadata

from onedocker.repository.onedocker_package import OneDockerPackageRepository
from onedocker.service.measurement import MeasurementService
from onedocker.service.metadata import MetadataService

DEFAULT_PROD_VERSION: str = "latest"
MEASUREMENT_TYPES: List[MeasurementType] = [MeasurementType.sha256]


class OneDockerRepositoryService:
    def __init__(
        self,
        storage_svc: StorageService,
        package_repository_path: str,
        metadata_svc: Optional[MetadataService] = None,
    ) -> None:
        self.package_repo = OneDockerPackageRepository(
            storage_svc, package_repository_path
        )
        self.metadata_svc = metadata_svc
        self.measurement_svc = MeasurementService()

    def upload(
        self,
        package_name: str,
        version: str,
        source: str,
    ) -> None:
        if not self._skip_version_validation_check(version):
            all_versions = self.package_repo.get_package_versions(package_name)
            if version in all_versions:
                raise ValueError(
                    f"Version {version} already exists. Please specify another version."
                )
        self.package_repo.upload(package_name, version, source)

        if self.metadata_svc:
            self.metadata_svc.put_metadata(
                metadata=self._generate_metadata(
                    package_name=package_name, version=version, source=source
                )
            )

    def download(self, package_name: str, version: str, destination: str) -> None:
        self.package_repo.download(package_name, version, destination)

    def _generate_measurements(self, source: str) -> Dict[MeasurementType, str]:
        measurements = self.measurement_svc.generate_measurements(
            measurement_types=MEASUREMENT_TYPES, file_path=source
        )
        return measurements

    def _generate_metadata(
        self,
        package_name: str,
        version: str,
        source: str,
    ) -> PackageMetadata:
        measurements = self._generate_measurements(source)
        return PackageMetadata(
            package_name=package_name, version=version, measurements=measurements
        )

    def archive_package(self, package_name: str, version: str) -> None:
        # TODO: Archive or delete checksum file associated with the archived package if exists.
        self.package_repo.archive_package(package_name, version)

    def _skip_version_validation_check(self, version: str) -> bool:
        # TODO: T129388192 remove this check when latest is no longer in use.
        if version == DEFAULT_PROD_VERSION:
            return True
        return False

    def get_package_measurements(
        self, package_name: str, version: str
    ) -> Dict[str, str]:
        if not self.metadata_svc:
            raise PcpError(
                "No MetadataService has been provided for OneDockerRepositoryService, which is required for get_package_measurements function"
            )

        md = self.metadata_svc.get_medadata(package_name=package_name, version=version)

        return {k.value: v for k, v in md.measurements.items()}



#!/usr/bin/env python3
# Copyright (c) Meta Platforms, Inc. and affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

"""
CLI for running an executable in OneDocker containers.


Usage:
    onedocker-runner <package_name> --version=<version> [options]

Options:
    -h --help                                               Show this help
    --repository_path=<repository_path>                     OneDocker repository path where the executables are downloaded from. No download when "LOCAL" repository is specified.
    --exe_path=<exe_path>                                   The local path where the executables are downloaded to.
    --exe_args=<exe_args>                                   The arguments the executable will use.
    --timeout=<timeout>                                     Set timeout (in sec) to kill the task.
    --log_path=<path>                                       Override the default path where logs are saved.
    --cert_params=<cert_params>                             String format of CertificateRequest dictionary if a TLS certificate is requested
    --verbose                                               Set logging level to DEBUG.
"""
import logging
import os
import resource
import stat
import subprocess
import sys
from pathlib import Path
from shlex import join, split
from typing import Optional

import psutil
import schema
from docopt import docopt
from fbpcp.entity.certificate_request import CertificateRequest

from fbpcp.service.storage_s3 import S3StorageService
from fbpcp.util.s3path import S3Path
from onedocker.common.env import ONEDOCKER_EXE_PATH, ONEDOCKER_REPOSITORY_PATH
from onedocker.common.util import run_cmd
from onedocker.repository.onedocker_repository_service import OneDockerRepositoryService


# The default OneDocker repository path on S3
DEFAULT_REPOSITORY_PATH = (
    "https://one-docker-repository-prod.s3.us-west-2.amazonaws.com/"
)

# The default path in the Docker image that is going to host the executables
DEFAULT_EXE_FOLDER = "/root/onedocker/package/"

# the default version of the binary
DEFAULT_BINARY_VERSION = "latest"

logger: logging.Logger


def _set_resource_limit() -> None:
    """Set the core file size for the child process"""
    core_size = resource.RLIM_INFINITY
    resource.setrlimit(resource.RLIMIT_CORE, (core_size, core_size))


def _build_executable_path(exe_path: str, exe_name: str) -> str:
    return f"{exe_path}{exe_name}"


def _prepare_executable(
    repository_path: str,
    exe_path: str,
    package_name: str,
    version: str,
) -> str:
    # package details
    exe_name = _parse_package_name(package_name)
    executable = _build_executable_path(exe_path, exe_name)

    # download executable from s3
    if repository_path.upper() != "LOCAL":
        _download_executables(repository_path, exe_path, package_name, version)

    else:
        logger.info("Local repository, skip download and attestation ...")

    # grant execute permission to the downloaded executable file
    if not os.access(executable, os.X_OK):
        os.chmod(executable, os.stat(executable).st_mode | stat.S_IEXEC)
    return executable


def _run_executable(
    executable: str, timeout: int, exe_args: Optional[str] = None
) -> None:
    # run execution cmd
    cmd = _build_cmd(executable, exe_args)

    logger.info(f"Running cmd: {cmd} ...")
    net_start = psutil.net_io_counters()

    return_code = run_cmd(cmd, timeout)
    if return_code != 0:
        logger.error(f"Subprocess returned non-zero return code: {return_code}")

    net_end = psutil.net_io_counters()
    logger.info(
        f"Net usage: {net_end.bytes_sent - net_start.bytes_sent} bytes sent, {net_end.bytes_recv - net_start.bytes_recv} bytes received"
    )

    sys.exit(return_code)


def _run_package(
    repository_path: str,
    exe_path: str,
    package_name: str,
    version: str,
    timeout: int,
    exe_args: Optional[str] = None,
    certificate_request: Optional[CertificateRequest] = None,
) -> None:
    logger.info(f"Starting to run {package_name}, version: {version}")
    executable = ""

    if certificate_request:
        _generate_certificate(certificate_request, exe_path)

    try:
        executable = _prepare_executable(
            repository_path=repository_path,
            exe_path=exe_path,
            package_name=package_name,
            version=version,
        )
    except Exception as err:
        logger.exception(
            f"An error was raised while preparing {package_name}:{version} from {repository_path}, error: {err}"
        )
        sys.exit(1)

    try:
        _run_executable(
            executable=executable,
            timeout=timeout,
            exe_args=exe_args,
        )
    except subprocess.TimeoutExpired as err:
        logger.exception(
            f"{timeout} seconds have passed. Now exiting the program...\n{err}"
        )
        sys.exit(1)
    except InterruptedError as err:
        logger.exception(
            f"Receive abort command from user, Now exiting the program...\n{err}",
        )
        sys.exit(1)
    except Exception as err:
        logger.exception(
            f"An error was raised while running {package_name}, error: {err}"
        )
        sys.exit(1)


def _build_cmd(executable: str, exe_args: Optional[str]) -> str:
    args_list = split(exe_args) if exe_args else []
    args_list.insert(0, executable)
    return join(args_list)


def _download_executables(
    repository_path: str,
    executable_path: str,
    package_name: str,
    version: str,
) -> None:
    exe_name = _parse_package_name(package_name)
    exe_local_path = executable_path + exe_name
    exe_s3_path = f"{repository_path}{package_name}/{version}/{exe_name}"
    # TODO: This is a short term solution. For long term, OneDocker Repository might need to take care.
    is_onedocker_package_unsigned_request = bool(
        os.environ.get("ONEDOCKER_PACKAGE_UNSIGNED_REQUEST", False)
    )
    storage_svc = S3StorageService(
        S3Path(repository_path).region,
        unsigned_enabled=is_onedocker_package_unsigned_request,
    )
    onedocker_repo_svc = OneDockerRepositoryService(storage_svc, repository_path)
    logger.info(f"Downloading package {package_name}: {version} from {exe_s3_path}")
    onedocker_repo_svc.download(package_name, version, exe_local_path)
    logger.info(
        f"Downloaded package {package_name}: {version} from {exe_s3_path} to {exe_local_path}"
    )


def _parse_package_name(package_name: str) -> str:
    # Some existing packages are like private_lift/lift, so we have to split it by slash
    return package_name.split("/")[-1]


def _read_config(
    config_name: str,
    argument: Optional[str],
    env_var: str,
    default_val: str,
) -> str:
    if argument:
        logger.info(f"Read {config_name} from program arguments...")
        return argument

    env_val = os.getenv(env_var)
    if env_val:
        logger.info(f"Read {config_name} from environment variables...")
        return env_val

    logger.info(f"Read {config_name} from default value...")
    return default_val


def _generate_certificate(
    certificate_request: CertificateRequest, exe_path: str
) -> Optional[str]:
    raise NotImplementedError(
        "Self signed Certificate is only available in development, CA certificate is pending for implementation."
    )


def main() -> None:
    global logger
    s = schema.Schema(
        {
            "<package_name>": str,
            "--version": str,
            "--repository_path": schema.Or(None, schema.And(str, len)),
            "--exe_path": schema.Or(None, schema.And(str, len)),
            "--exe_args": schema.Or(None, schema.And(str, len)),
            "--timeout": schema.Or(None, schema.Use(int)),
            "--log_path": schema.Or(None, schema.Use(Path)),
            "--cert_params": schema.Or(None, schema.And(str, len)),
            "--verbose": bool,
            "--help": bool,
        }
    )

    arguments = s.validate(docopt(__doc__))

    logger = logging.getLogger(__name__)
    log_path = arguments["--log_path"]
    log_level = logging.DEBUG if arguments["--verbose"] else logging.INFO
    logging.basicConfig(filename=log_path, level=log_level)

    repository_path = _read_config(
        "repository_path",
        arguments["--repository_path"],
        ONEDOCKER_REPOSITORY_PATH,
        DEFAULT_REPOSITORY_PATH,
    )
    exe_path = _read_config(
        "exe_path",
        arguments["--exe_path"],
        ONEDOCKER_EXE_PATH,
        DEFAULT_EXE_FOLDER,
    )
    certificate_request = (
        CertificateRequest.create_instance(arguments["--cert_params"])
        if arguments["--cert_params"]
        else None
    )

    _run_package(
        repository_path=repository_path,
        exe_path=exe_path,
        package_name=arguments["<package_name>"],
        version=arguments["--version"],
        timeout=arguments["--timeout"],
        exe_args=arguments["--exe_args"],
        certificate_request=certificate_request,
    )


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
# Copyright (c) Meta Platforms, Inc. and affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

# patternlint-disable f-string-may-be-missing-leading-f

from typing import TypeVar


FIREWALL_RULE_INITIAL_PORT = 5000
FIREWALL_RULE_FINAL_PORT = 15500


AvailabilityZone = TypeVar("AvailabilityZone", str, bytes)


CONTAINER_CPU = 4096
CONTAINER_MEMORY = 30720
CONTAINER_IMAGE = "539290649537.dkr.ecr.us-west-2.amazonaws.com/one-docker-prod:latest"
TASK_POLICY = {
    "Version": "2012-10-17",
    "Statement": [
        {"Effect": "Allow", "Action": ["s3:*", "s3-object-lambda:*"], "Resource": "*"}
    ],
}

IGW_ROUTE_TARGET_PREFIX: str = "igw-"
IGW_ROUTE_DESTINATION_CIDR_BLOCK: str = "0.0.0.0/0"

DEFAULT_VPC_CIDR = "10.0.0.0/16"
DEFAULT_PARTNER_VPC_CIDR = "10.1.0.0/16"



#!/usr/bin/env python3
# Copyright (c) Meta Platforms, Inc. and affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict
import glob
import os
from typing import Any, Dict, List, Optional

from fbpcp.entity.file_information import FileInfo
from fbpcp.entity.policy_statement import PolicyStatement, PublicAccessBlockConfig
from fbpcp.gateway.gcs import GCSGateway
from fbpcp.service.storage import PathType, StorageService
from fbpcp.util.gcspath import GCSPath


class GCSStorageService(StorageService):
    def __init__(
        self,
        credentials_json: Optional[str] = None,
        config: Optional[Dict[str, Any]] = None,
    ) -> None:
        self.gcs_gateway = GCSGateway(credentials_json, config)

    def __check_dir(self, local_dir: str) -> None:
        if not os.path.exists(local_dir):
            raise ValueError("directory does not exist: " + local_dir)

    def read(self, filename: str) -> str:
        """Read a file data

        Args:
            filename: fully qualified GCS filename (ex: "https://storage.cloud.google.com/bucket-name/key-name")

        Returns:
            return: file contents as str
        """
        gcs_path = GCSPath(filename)
        return self.gcs_gateway.get_object(gcs_path.bucket, gcs_path.key)

    def write(self, filename: str, data: str) -> None:
        """Write data into a file

        Args:
            filename: fully qualified GCS filename (ex: "https://storage.cloud.google.com/bucket-name/key-name")
            data: file contents as str
        """
        gcs_path = GCSPath(filename)
        return self.gcs_gateway.put_object(gcs_path.bucket, gcs_path.key, data)

    def copy(self, source: str, destination: str, recursive: bool = False) -> None:
        """Move a file or folder between local storage and GCS, or between GCS and GCS

        Args:
            source: source file
            destination: destination file
            recursive: whether to recursively copy a folder

        Raises:
            ValueError: Source is folder and recursive is False
            ValueError: Source is not a valid Local or GCS path
            ValueError: Destination is not a valid Local or GCS path
            ValueError: Both source and destination are the same
            ValueError: Both source and destination are local
        """
        sourceType = StorageService.path_type(source)
        destinationType = StorageService.path_type(destination)
        if sourceType != PathType.GCS and sourceType != PathType.Local:
            raise ValueError("Source is not a valid Local or GCS path")
        if destinationType != PathType.GCS and destinationType != PathType.Local:
            raise ValueError("Destination is not a valid Local or GCS path")
        if source == destination:
            raise ValueError("Both source and destination are the same")

        # source is Local
        if sourceType == PathType.Local:
            if destinationType == PathType.Local:
                raise ValueError("Both source and destination are local files")
            else:
                gcs_path = GCSPath(destination)
                if recursive:
                    self.upload_dir(
                        source=source,
                        gcs_path_bucket=gcs_path.bucket,
                        gcs_path_key=gcs_path.key,
                    )
                else:
                    if os.path.isdir(source):
                        raise ValueError(
                            f"Source {source} is a folder. Use --recursive"
                        )
                    self.gcs_gateway.upload_file(
                        file_name=source, bucket=gcs_path.bucket, key=gcs_path.key
                    )
        # source is GCS
        elif sourceType == PathType.GCS:
            source_gcs_path = GCSPath(source)
            if destinationType == PathType.Local:
                if recursive:
                    self.download_dir(
                        gcs_path_bucket=source_gcs_path.bucket,
                        gcs_path_key=source_gcs_path.key,
                        destination=destination,
                    )
                else:
                    self.gcs_gateway.download_file(
                        bucket=source_gcs_path.bucket,
                        key=source_gcs_path.key,
                        filename=destination,
                    )
            elif destinationType == PathType.GCS:
                destination_gcs_path = GCSPath(destination)
                if recursive:
                    self.copy_dir(
                        source_bucket=source_gcs_path.bucket,
                        source_key=source_gcs_path.key,
                        destination_bucket=destination_gcs_path.bucket,
                        destination_key=destination_gcs_path.key,
                    )
                else:
                    self.gcs_gateway.copy(
                        source_bucket_name=source_gcs_path.bucket,
                        source_key=source_gcs_path.key,
                        dest_bucket_name=destination_gcs_path.bucket,
                        dest_key=destination_gcs_path.key,
                    )

    def upload_dir(self, source: str, gcs_path_bucket: str, gcs_path_key: str) -> None:
        """Upload a directory from the filesystem to GCS

        Args:
            source: source directory (local)
            gcs_path_bucket: GCS bucket
            gcs_path_key: GCS key
        """
        # Check that source directory exists
        self.__check_dir(source)

        # Get list of files
        rel_paths = glob.glob(source + "/**", recursive=True)
        for local_file in rel_paths:
            if os.path.isfile(local_file):
                remote_path = local_file.replace(source, gcs_path_key, 1)
                self.gcs_gateway.upload_file(
                    file_name=local_file, bucket=gcs_path_bucket, key=remote_path
                )

    def download_dir(
        self, gcs_path_bucket: str, gcs_path_key: str, destination: str
    ) -> None:
        """Recursively downloads a directory from GCS to the filesystem

        Args:
            gcs_path_bucket: GCS bucket
            gcs_path_key: GCS key
            destination: destination directory (local)
        """
        # Get list of files
        blob_names = self.gcs_gateway.list_objects(
            bucket=gcs_path_bucket, key=gcs_path_key
        )
        for blob_name in blob_names:
            if blob_name.endswith("/"):
                continue
            file_split = destination.split("/") + blob_name.replace(
                gcs_path_key, "", 1
            ).split("/")
            dir_name = "/".join(file_split[0:-1])
            if not os.path.exists(dir_name):
                os.makedirs(dir_name)
            file_name = "/".join(file_split)
            self.gcs_gateway.download_file(
                bucket=gcs_path_bucket, key=blob_name, file_name=file_name
            )

    def copy_dir(
        self,
        source_bucket: str,
        source_key: str,
        destination_bucket: str,
        destination_key: str,
    ) -> None:
        """Copy a directory from GCS to another directory in GCS

        Args:
            source_bucket: Source GCS bucket
            source_key: Source GCS key (directory)
            destination_bucket: Destination GCS bucket
            destination_key: Destination GCS key (directory)

        Raises:
            ValueError: Source and Destination are the same
        """
        if source_bucket == destination_bucket and source_key == destination_key:
            raise ValueError("Source and Destination are the same")

        # Get list of files
        src_blob_names = self.gcs_gateway.list_objects(
            bucket=source_bucket, key=source_key
        )
        dest_key_split = destination_key.split("/")
        for src_blob_name in src_blob_names:
            if src_blob_name.endswith("/"):
                continue
            dest_file_name = "/".join(
                dest_key_split + src_blob_name.replace(source_key, "", 1).split("/")
            )
            self.gcs_gateway.copy(
                source_bucket, src_blob_name, destination_bucket, dest_file_name
            )

    def delete(self, filename: str) -> None:
        """Delete a GCS file

        Args:
            filename: fully qualified GCS filename to be deleted (ex: "https://storage.cloud.google.com/bucket-name/key-name")
        """
        gcs_path = GCSPath(filename)
        return self.gcs_gateway.delete_object(gcs_path.bucket, gcs_path.key)

    def file_exists(self, filename: str) -> bool:
        """Check existence of a GCS file

        Args:
            filename: fully qualified GCS filename to be checked (ex: "https://storage.cloud.google.com/bucket-name/key-name")
        """
        gcs_path = GCSPath(filename)
        return self.gcs_gateway.object_exists(gcs_path.bucket, gcs_path.key)

    def get_file_info(self, filename: str) -> FileInfo:
        """Get file information

        Args:
            filename: fully qualified GCS filename to be inspected (ex: "https://storage.cloud.google.com/bucket-name/key-name")

        Returns:
            FileInfo: file_name, file_size, last_modified
        """
        gcs_path = GCSPath(filename)
        file_info = self.gcs_gateway.get_object_info(gcs_path.bucket, gcs_path.key)
        return FileInfo(
            file_name=filename,
            last_modified=file_info.get("updated"),
            file_size=file_info.get("size"),
        )

    def get_file_size(self, filename: str) -> int:
        """Get file size

        Args:
            filename: fully qualified GCS filename to be inspected (ex: "https://storage.cloud.google.com/bucket-name/key-name")

        Returns:
            int: file size (in bytes)
        """
        gcs_path = GCSPath(filename)
        return self.gcs_gateway.get_object_size(gcs_path.bucket, gcs_path.key)

    def list_folders(self, filename: str) -> List[str]:
        """List folders

        Args:
            filename: fully qualified GCS filename to be inspected (ex: "https://storage.cloud.google.com/bucket-name/key-name")

        Returns:
            List[str]: List of folders found in filename
        """
        folders = []
        gcs_path = GCSPath(filename)
        blob_names = self.gcs_gateway.list_objects(
            bucket=gcs_path.bucket, key=gcs_path.key
        )
        for blob_name in blob_names:
            if blob_name.endswith("/"):
                folders.append(blob_name)
        return folders

    def get_bucket_policy_statements(self, bucket: str) -> List[PolicyStatement]:
        raise NotImplementedError

    def get_bucket_public_access_block(self, bucket: str) -> PublicAccessBlockConfig:
        raise NotImplementedError

    def list_files(self, dirPath: str) -> List[str]:
        raise NotImplementedError



#!/usr/bin/env python3
# Copyright (c) Meta Platforms, Inc. and affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

# patternlint-disable f-string-may-be-missing-leading-f
from enum import Enum
from typing import List


class ValidationStepNames(Enum):
    """
    Enumerates the names of validation steps, each step corresponding to a
    method of pce.validator.ValidationSuite called validate_{code_name}
    """

    VPC_CIDR = ("VPC CIDR", "vpc_cidr")
    VPC_PEERING = ("VPC peering", "vpc_peering")
    FIREWALL = ("Firewall", "firewall")
    ROUTE_TABLE = ("Route table", "route_table")
    SUBNETS = ("Subnets", "subnets")
    CLUSTER_DEFINITION = ("Cluster definition", "cluster_definition")
    IAM_ROLES = ("IAM roles", "iam_roles")
    LOG_GROUP = ("Log group", "log_group")

    def __init__(self, formatted_name: str, code_name: str) -> None:
        """
        set the Enum member values under better attribute names for
        easier access later eg ValidationStepNames.VPC_CIDR.code_name
        """
        self.formatted_name = formatted_name
        self.code_name = code_name

    @staticmethod
    def code_names() -> List[str]:
        return [step.code_name for step in ValidationStepNames]

    @staticmethod
    def from_code_name(code_name: str) -> "ValidationStepNames":
        for step_name in ValidationStepNames:
            if step_name.code_name == code_name:
                return step_name
        raise ValueError(
            f"No ValidationStepName member exists with code_name={code_name}"
        )



#!/usr/bin/env python3
# Copyright (c) Meta Platforms, Inc. and affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

import json
from dataclasses import dataclass, fields
from enum import Enum
from typing import List, Optional

from fbpcp.error.pcp import InvalidParameterError


class KeyAlgorithm(Enum):
    # For now we only support RSA since it is one of the most common algorithms, will add more key algorithms in the future
    RSA = "RSA"

    @classmethod
    def has_member(cls, name: str) -> bool:
        return name in cls.__members__


@dataclass
class CertificateRequest:
    key_algorithm: KeyAlgorithm
    key_size: int
    passphrase: Optional[str]
    cert_folder: Optional[str]
    private_key_name: Optional[str]
    certificate_name: Optional[str]
    days_valid: Optional[int]
    country_name: Optional[str]
    state_or_province_name: Optional[str]
    locality_name: Optional[str]
    organization_name: Optional[str]
    common_name: Optional[str]
    dns_name: Optional[str]

    @classmethod
    def get_non_optional_fields(cls) -> List[str]:
        fields_list = fields(cls)
        return [
            field.name for field in fields_list if field.type != Optional[field.type]
        ]

    @classmethod
    def create_instance(cls, cert_params: str) -> "CertificateRequest":
        try:
            cert_params_dict = json.loads(cert_params)
        except Exception as err:
            raise InvalidParameterError(
                f"An error was raised when unpack cert_params for CertificateRequest {cert_params}: {err}"
            )
        required_keys = set(cls.get_non_optional_fields())
        if not required_keys.issubset(set(cert_params_dict.keys())):
            raise InvalidParameterError(
                f"One of the required parameters {required_keys} is missing in cert_params: {cert_params_dict.keys()}"
            )
        key_algorithm_str = cert_params_dict["key_algorithm"]
        if not KeyAlgorithm.has_member(key_algorithm_str):
            raise InvalidParameterError(
                f"key_algorithm {key_algorithm_str} is not supported"
            )

        return cls(
            key_algorithm=KeyAlgorithm[key_algorithm_str],
            key_size=int(cert_params_dict["key_size"]),
            passphrase=cert_params_dict["passphrase"],
            cert_folder=cert_params_dict.get("cert_folder", None),
            private_key_name=cert_params_dict.get("private_key_name", None),
            certificate_name=cert_params_dict.get("certificate_name", None),
            days_valid=cert_params_dict.get("days_valid", None),
            country_name=cert_params_dict.get("country_name", None),
            state_or_province_name=cert_params_dict.get("state_or_province_name", None),
            locality_name=cert_params_dict.get("locality_name", None),
            organization_name=cert_params_dict.get("organization_name", None),
            common_name=cert_params_dict.get("common_name", None),
            dns_name=cert_params_dict.get("dns_name", None),
        )

    def convert_to_cert_params(self) -> str:
        cert_dict = {}
        for f in fields(self.__class__):
            fname = f.name
            fvalue = getattr(self, f.name)
            if fname == "key_algorithm":
                cert_dict[fname] = fvalue.value
            elif fvalue is not None:
                cert_dict[fname] = fvalue

        return json.dumps(cert_dict)



#!/usr/bin/env python3
# Copyright (c) Meta Platforms, Inc. and affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

from typing import Any, Dict, Optional

import boto3
from boto3.dynamodb.types import TypeDeserializer, TypeSerializer
from botocore.exceptions import ClientError
from fbpcp.decorator.error_handler import error_handler
from fbpcp.error.pcp import PcpError
from fbpcp.gateway.aws import AWSGateway


class DynamoDBGateway(AWSGateway):
    def __init__(
        self,
        region: str,
        access_key_id: Optional[str] = None,
        access_key_data: Optional[str] = None,
        config: Optional[Dict[str, Any]] = None,
    ) -> None:
        super().__init__(region, access_key_id, access_key_data, config)
        # pyre-ignore
        self.client = boto3.client("dynamodb", region_name=self.region, **self.config)
        self.serializer = TypeSerializer()
        self.deserializer = TypeDeserializer()

    @error_handler
    def put_item(self, table_name: str, item: Dict[str, Any]) -> None:
        dynamo_item = {k: self.serializer.serialize(v) for k, v in item.items()}
        try:
            self.client.put_item(
                TableName=table_name,
                Item=dynamo_item,
                ConditionExpression="attribute_not_exists(instance_id)",
            )
        except ClientError as e:
            if e.response["Error"]["Code"] == "ConditionalCheckFailedException":
                raise PcpError(
                    "Cannot put item, an item with the given instance_id already exists"
                )
            raise e

    @error_handler
    def get_item(
        self,
        table_name: str,
        key_name: str,
        # pyre-ignore
        key_value: Any,
    ) -> Dict[str, Any]:
        key = {key_name: self.serializer.serialize(key_value)}
        raw_response = self.client.get_item(
            TableName=table_name,
            Key=key,
        )
        if "Item" not in raw_response:
            raise PcpError(f"No item in database for key {key_name}: {key_value}")
        return {
            k: self.deserializer.deserialize(v) for k, v in raw_response["Item"].items()
        }

    @error_handler
    def update_item(
        self,
        table_name: str,
        key_name: str,
        # pyre-ignore
        key_value: Any,
        attribute_name: str,
        # pyre-ignore
        new_value: Any,
    ) -> None:
        update_expression = f"SET {attribute_name} = :new_data"
        expression_attributes = {":new_data": new_value}
        boto3_key = {key_name: self.serializer.serialize(key_value)}
        boto3_attributes = {
            k: self.serializer.serialize(v) for k, v in expression_attributes.items()
        }
        try:
            self.client.update_item(
                TableName=table_name,
                Key=boto3_key,
                UpdateExpression=update_expression,
                ExpressionAttributeValues=boto3_attributes,
                ConditionExpression="attribute_exists(instance_id)",
            )
        except ClientError as e:
            if e.response["Error"]["Code"] == "ConditionalCheckFailedException":
                raise PcpError(
                    "Cannot get item, an item with the given instance_id does not already exist"
                )
            raise e

    @error_handler
    def delete_item(
        self,
        table_name: str,
        key_name: str,
        # pyre-ignore
        key_value: Any,
    ) -> None:
        boto3_key = {key_name: self.serializer.serialize(key_value)}
        self.client.delete_item(TableName=table_name, Key=boto3_key)



#!/usr/bin/env python3
# Copyright (c) Meta Platforms, Inc. and affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

import abc
from typing import Dict, List, Optional, Union

from fbpcp.entity.cluster_instance import Cluster

from fbpcp.entity.container_instance import ContainerInstance
from fbpcp.entity.container_type import ContainerType
from fbpcp.error.pcp import PcpError


class ContainerService(abc.ABC):
    @abc.abstractmethod
    def get_region(
        self,
    ) -> str:
        pass

    @abc.abstractmethod
    def get_cluster(
        self,
    ) -> str:
        pass

    @abc.abstractmethod
    def create_instance(
        self,
        container_definition: str,
        cmd: str,
        env_vars: Optional[Dict[str, str]] = None,
        container_type: Optional[ContainerType] = None,
    ) -> ContainerInstance:
        pass

    @abc.abstractmethod
    def create_instances(
        self,
        container_definition: str,
        cmds: List[str],
        env_vars: Optional[Union[Dict[str, str], List[Dict[str, str]]]] = None,
        container_type: Optional[ContainerType] = None,
    ) -> List[ContainerInstance]:
        pass

    @abc.abstractmethod
    def get_instance(self, instance_id: str) -> Optional[ContainerInstance]:
        """Get a specific container instance.

        Args:
            instance_id: uniquely identify a container instance.

        Returns:
            Optional - a container instance if the specified container instances was found, a None otherwise.

        Raises:
            PcpError: The specified container instance wasn't found in the cluster.
        """
        pass

    @abc.abstractmethod
    def get_instances(
        self, instance_ids: List[str]
    ) -> List[Optional[ContainerInstance]]:
        """Get one or more container instances.

        Args:
            instance_ids: the instance ids of the container instances.

        Returns:
            A list of Optional, in the same order as the input ids. For example, if
            users pass 3 instance_ids and the second instance could not be found,
            then returned list should also have 3 elements, with the 2nd elements being None.
        """
        pass

    @abc.abstractmethod
    def cancel_instance(self, instance_id: str) -> None:
        """Cancel a running container instance.

        cancel_instance does not gurantee immediate terminations. Some implementations
        may wait for a certain grace period.
        Cancelled instance may be removed immediately, or after a short time.

        Args:
            instance_id: the instance id of the container instance to cancel.
        """
        pass

    @abc.abstractmethod
    def cancel_instances(self, instance_ids: List[str]) -> List[Optional[PcpError]]:
        """Cancel one or more running container instances.

        Args:
            instance_id: the instance ids of the container instances to cancel.

        Returns:
            A list of Optionals, in the same order as the input instance ids. A `None` indicates
            a successful cancellation, whereas a `PceError` indicates a failed cancellation and
            describes the error reason.
        """
        pass

    @abc.abstractmethod
    def get_current_instances_count(self) -> int:
        """Get total pending and running instances count for cluster
        Returns:
            Integer that represent the total pending and running instances count for cluster
        """
        pass

    @abc.abstractmethod
    def get_cluster_instance(self) -> Cluster:
        """Get cluster instance
        Returns:
            Cluster that represent a cluster instance
        """
        pass



#!/usr/bin/env python3
# Copyright (c) Meta Platforms, Inc. and affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict
from typing import Any, Dict

from onedocker.entity.measurement import MeasurementType

from onedocker.entity.metadata import PackageMetadata


def map_dynamodbitem_to_packagemetadata(
    dynamodb_item: Dict[str, Any]
) -> PackageMetadata:
    measurements = {
        MeasurementType(key): value
        for key, value in dynamodb_item.get("measurements", {}).items()
    }
    return PackageMetadata(
        package_name=dynamodb_item.get("package_name"),
        version=dynamodb_item.get("version"),
        measurements=measurements,
    )



# Copyright (c) Meta Platforms, Inc. and affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

from typing import Any, Dict, List, Optional

import boto3
from fbpcp.decorator.error_handler import error_handler
from fbpcp.entity.vpc_peering import VpcPeering
from fbpcp.gateway.aws import AWSGateway
from fbpcp.mapper.aws import map_ec2vpcpeering_to_vpcpeering


class EC2Gateway(AWSGateway):
    def __init__(
        self,
        region: str,
        access_key_id: Optional[str] = None,
        access_key_data: Optional[str] = None,
        config: Optional[Dict[str, Any]] = None,
    ) -> None:
        super().__init__(region, access_key_id, access_key_data, config)

        # pyre-ignore
        self.client = boto3.client("ec2", region_name=self.region, **self.config)

    def describe_availability_zones(
        self,
        tags: Optional[Dict[str, str]] = None,
    ) -> List[str]:
        response = self.client.describe_availability_zones()

        return [aws_az["ZoneName"] for aws_az in response["AvailabilityZones"]]

    def describe_vpc_peering_connections_with_accepter_vpc_id(
        self,
        vpc_id: str,
    ) -> Optional[List[VpcPeering]]:
        filters = [
            {"Name": "accepter-vpc-info.vpc-id", "Values": [vpc_id]},
        ]

        response = self.client.describe_vpc_peering_connections(Filters=filters)
        return (
            [
                map_ec2vpcpeering_to_vpcpeering(vpc_conn, vpc_id)
                for vpc_conn in response["VpcPeeringConnections"]
            ]
            if response["VpcPeeringConnections"]
            else None
        )

    @error_handler
    def accept_vpc_peering_connection(
        self, vpc_peering_connection_id: str, vpc_id: str
    ) -> Optional[VpcPeering]:

        response = self.client.accept_vpc_peering_connection(
            VpcPeeringConnectionId=vpc_peering_connection_id
        )

        return (
            map_ec2vpcpeering_to_vpcpeering(response["VpcPeeringConnection"], vpc_id)
            if response["VpcPeeringConnection"]
            else None
        )

    @error_handler
    def create_route(
        self, route_table_id: str, vpc_peering_connection_id: str, dest_cidr: str
    ) -> bool:

        response = self.client.create_route(
            RouteTableId=route_table_id,
            DestinationCidrBlock=dest_cidr,
            VpcPeeringConnectionId=vpc_peering_connection_id,
        )

        return response["Return"]

    @error_handler
    def replace_route(
        self, route_table_id: str, vpc_peering_connection_id: str, dest_cidr: str
    ) -> None:

        self.client.replace_route(
            RouteTableId=route_table_id,
            DestinationCidrBlock=dest_cidr,
            VpcPeeringConnectionId=vpc_peering_connection_id,
        )



#!/usr/bin/env python3
# Copyright (c) Meta Platforms, Inc. and affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

from fbpcp.error.pcp import InvalidParameterError, PcpError, ThrottlingError
from google.cloud.exceptions import GoogleCloudError


# reference: https://gcloud.readthedocs.io/en/latest/_modules/google/cloud/exceptions.html
def map_gcp_error(error: GoogleCloudError) -> PcpError:
    code = error.code
    message = error.message
    if code == 429:
        return ThrottlingError(message)
    if code == 400:
        return InvalidParameterError(message)
    return PcpError(message)



#!/usr/bin/env python3
# Copyright (c) Meta Platforms, Inc. and affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

from dataclasses import dataclass, field
from typing import Any, Dict

from onedocker.entity.measurement import MeasurementType


@dataclass
class PackageMetadata:
    package_name: str
    version: str
    measurements: Dict[MeasurementType, str] = field(default_factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        return {
            "package_name": self.package_name,
            "version": self.version,
            "measurements": {k.value: v for k, v in self.measurements.items()},
        }



#!/usr/bin/env python3
# Copyright (c) Meta Platforms, Inc. and affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict
from dataclasses import dataclass

from enum import Enum

from dataclasses_json import dataclass_json


class MeasurementType(Enum):
    sha256 = "sha256"
    sha512 = "sha512"

    @classmethod
    def has_member(cls, name: str) -> bool:
        return name in cls.__members__


@dataclass_json
@dataclass
class Measurement:
    key: MeasurementType
    value: str



#!/usr/bin/env python3
# Copyright (c) Meta Platforms, Inc. and affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

import re
from typing import Tuple


class GCSPath:
    bucket: str
    key: str

    def __init__(self, fileURL: str) -> None:
        self.bucket, self.key = self._get_bucket_key(fileURL)

    def __eq__(self, other: "GCSPath") -> bool:
        return self.bucket == other.bucket and self.key == other.key

    # virtual host style url
    # https://storage.cloud.google.com/<bucket>/<key>
    def _get_bucket_key(self, fileURL: str) -> Tuple[str, str]:
        match = re.search("^https?://storage.cloud.google.com/(.*)$", fileURL)
        if not match:
            raise ValueError(f"Could not parse {fileURL} as an GCSPath")
        bucket, *rest = match.group(1).split("/")
        key = "/".join(rest)
        return (bucket, key)



# Copyright (c) Meta Platforms, Inc. and affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

"""


Usage:
    pce_validator --region=<region> --pce-id=<pce_id> [--key-id=<key_id>] [--key-data=<key_data>] [--role=<role>] [--run-step=<run-step> | --skip-step=<skip-step>]...

Options ([+] can be repeated):
    --region=<region>               (AWS) Region name
    --key-id=<key_id>               Key id
    --key-data=<key_data>           Key data
    --pce-id=<pce_id>               PCE id
    --role=<role>                   publisher, partner
    --run-step=<run-step> [+]       name of a validation step to be explicitly run, eg vpc_cidr
    --skip-step=<skip-step> [+]     name of a validation step to be skipped, eg vpc_peering
"""


import logging
import sys
from enum import Enum
from typing import List

from docopt import docopt
from fbpcp.service.pce_aws import AWSPCEService
from pce.entity.mpc_roles import MPCRoles
from pce.gateway.sts import STSGateway
from pce.validator.duplicate_pce_resources_checker import DuplicatePCEResourcesChecker
from pce.validator.message_templates.validator_step_names import ValidationStepNames
from pce.validator.validation_suite import ValidationSuite
from schema import And, Optional, Or, Schema, Use


# The main method in this module explicitly sets an exit code to indicate
# success or non-success of the validation, but has no further classification
# of errors and corresponding exit codes. Invalid CLI args also result in a
# exitcode of 1, either explicitly via sys.exit("message") or implicitly due
# to schema.SchemaError exception raised by schema.Schema.validate().
class ValidatorResult(Enum):
    SUCCESS = 0
    ERROR = 1


OVERALL_SUCCESS_MESSAGE = "Your PCE environments are set up correctly."
INCOMPATIBLE_STEP_ARG_ERROR_MESSAGE = (
    "Only either of --run-step or --skip-step should be provided"
)


def get_arn(
    region: str,
    key_id: str,
    key_data: str,
) -> str:

    sts_gateway: STSGateway = STSGateway(region, key_id, key_data, None)
    return sts_gateway.get_caller_arn()


def validate_pce(
    region: str,
    key_id: str,
    key_data: str,
    pce_id: str,
    role: MPCRoles,
    skip_steps: List[ValidationStepNames],
    run_steps: List[ValidationStepNames],
) -> ValidatorResult:
    duplicate_resource_checker = DuplicatePCEResourcesChecker(
        region, key_id, key_data, None
    )
    duplicate_resources = duplicate_resource_checker.check_pce(pce_id)
    if duplicate_resources:
        logging.error(
            f"Failed to load PCE due to duplicate resources tagged under same "
            "pce id. Only one each of these resources can be tagged with the "
            f"pce:pce-id ({pce_id}), and the others are mistagged. Look at "
            "other properties of these resources (like id) for a hint to the "
            "pce:pce-id the resource may correctly belong to. Details follow:"
        )
        for duplicate_resource in duplicate_resources:
            logging.error(
                f"Multiple {duplicate_resource.resource_name_plural} tagged "
                f"with pce:pce-id ({pce_id}): {duplicate_resource.duplicate_resource_ids}"
            )
        return ValidatorResult.ERROR

    pce_service = AWSPCEService(region, key_id, key_data, None)
    logging.info(f"Loading the PCE {pce_id}...")
    pce = pce_service.get_pce(pce_id)
    logging.info(f"PCE loaded: {pce}")

    arn = get_arn(region, key_id, key_data)
    logging.info(f"ARN: {arn}")

    validator = ValidationSuite(region, key_id, key_data, None, role)

    failed_results = validator.validate_network_and_compute(pce, skip_steps, run_steps)
    if failed_results:
        logging.error(
            f"Validation failed for PCE {pce_id}:\n{ValidationSuite.summarize_errors(failed_results)}"
        )
        # check if any error level results, to set overall result as ValidatorResult.ERROR
        # warning level results don't set overall result as ValidatorResult.ERROR
        if ValidationSuite.contains_error_result(failed_results):
            return ValidatorResult.ERROR
    else:
        print(OVERALL_SUCCESS_MESSAGE)
    return ValidatorResult.SUCCESS


def main() -> None:
    logging.basicConfig(level=logging.INFO)

    s = Schema(
        {
            "--region": str,
            "--pce-id": str,
            Optional("--key-id"): Or(None, str),
            Optional("--key-data"): Or(None, str),
            Optional("--role"): Or(
                None,
                And(
                    Use(str.upper),
                    lambda s: s in ("PUBLISHER", "PARTNER"),
                    Use(MPCRoles),
                ),
            ),
            Optional("--run-step"): Or(
                None,
                And(
                    list,
                    lambda step_list: all(
                        [step in ValidationStepNames.code_names() for step in step_list]
                    ),
                ),
            ),
            Optional("--skip-step"): Or(
                None,
                And(
                    list,
                    lambda step_list: all(
                        [step in ValidationStepNames.code_names() for step in step_list]
                    ),
                ),
            ),
        }
    )

    arguments = s.validate(docopt(__doc__))

    region = arguments["--region"]
    key_id = arguments["--key-id"]
    key_data = arguments["--key-data"]
    pce_id = arguments["--pce-id"]
    role = arguments["--role"]
    skip_step_code_names = arguments["--skip-step"]
    run_step_code_names = arguments["--run-step"]

    if run_step_code_names and skip_step_code_names:
        sys.exit(INCOMPATIBLE_STEP_ARG_ERROR_MESSAGE)

    skip_steps = [
        ValidationStepNames.from_code_name(code_name)
        for code_name in skip_step_code_names
    ]
    run_steps = [
        ValidationStepNames.from_code_name(code_name)
        for code_name in run_step_code_names
    ]

    result = validate_pce(region, key_id, key_data, pce_id, role, skip_steps, run_steps)
    # set program's exitcode to the value defined in the ValidatorResult enum
    sys.exit(result.value)


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
# Copyright (c) Meta Platforms, Inc. and affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

import logging
from typing import Any, Dict, List, Optional, Union

from fbpcp.entity.cloud_provider import CloudProvider

from fbpcp.entity.cluster_instance import Cluster
from fbpcp.entity.container_instance import ContainerInstance
from fbpcp.entity.container_type import ContainerType, ContainerTypeConfig
from fbpcp.error.pcp import PcpError
from fbpcp.gateway.ecs import ECSGateway
from fbpcp.metrics.emitter import MetricsEmitter
from fbpcp.service.container import ContainerService
from fbpcp.util.aws import split_container_definition

AWS_API_INPUT_SIZE_LIMIT = 100  # AWS API Call Capacity Limit


class AWSContainerService(ContainerService):
    def __init__(
        self,
        region: str,
        cluster: str,
        subnets: Optional[List[str]] = None,
        access_key_id: Optional[str] = None,
        access_key_data: Optional[str] = None,
        config: Optional[Dict[str, Any]] = None,
        metrics: Optional[MetricsEmitter] = None,
        session_token: Optional[str] = None,
    ) -> None:
        self.logger: logging.Logger = logging.getLogger(__name__)
        self.region = region
        self.cluster = cluster
        self.subnets = subnets
        self.ecs_gateway = ECSGateway(
            region, access_key_id, access_key_data, config, metrics, session_token
        )

    def get_region(
        self,
    ) -> str:
        return self.region

    def get_cluster(
        self,
    ) -> str:
        return self.cluster

    def create_instance(
        self,
        container_definition: str,
        cmd: str,
        env_vars: Optional[Dict[str, str]] = None,
        container_type: Optional[ContainerType] = None,
    ) -> ContainerInstance:
        task_definition, container = split_container_definition(container_definition)

        if not self.subnets:
            raise PcpError(
                "No subnets specified. It's required to create container instances."
            )
        cpu = None
        memory = None
        if container_type is not None:
            container_config = ContainerTypeConfig.get_config(
                CloudProvider.AWS, container_type
            )
            cpu, memory = container_config.cpu, container_config.memory
        return self.ecs_gateway.run_task(
            task_definition=task_definition,
            container=container,
            cmd=cmd,
            cluster=self.cluster,
            subnets=self.subnets,
            env_vars=env_vars,
            cpu=cpu,
            memory=memory,
        )

    def create_instances(
        self,
        container_definition: str,
        cmds: List[str],
        env_vars: Optional[Union[Dict[str, str], List[Dict[str, str]]]] = None,
        container_type: Optional[ContainerType] = None,
    ) -> List[ContainerInstance]:
        """
        Args:
            container_definition: a string representing the container definition.
            cmds: A list of cmds per instance to run inside each instance.
            env_vars: A dictionary or list of dictionaries of env_vars to be set in instances.
            When it is a single dictionary, all env vars in the dict will be set in all
            instances. When it is a list of dicts, it is expected that the length of the list
            is the same as the length of the cmds list, such that each item corresponds
            to one instance.
            container_type: The type of container to create.
        """
        if type(env_vars) is list and len(env_vars) != len(cmds):
            raise ValueError(
                f"Length of env_vars list {len(env_vars)} is different from length of cmds {len(cmds)}."
            )

        instances = [
            self.create_instance(
                container_definition=container_definition,
                cmd=cmds[i],
                env_vars=env_vars[i] if type(env_vars) is list else env_vars,
                container_type=container_type,
            )
            for i in range(len(cmds))
        ]

        self.logger.info(
            f"AWSContainerService created {len(instances)} containers successfully"
        )
        return instances

    def get_instance(self, instance_id: str) -> Optional[ContainerInstance]:
        return self.ecs_gateway.describe_task(self.cluster, instance_id)

    def get_instances(
        self, instance_ids: List[str]
    ) -> List[Optional[ContainerInstance]]:
        """Get one or more container instances

        Args:
            instance_ids: a list of the container instances.

        Returns:
            A list of Optional, in the same order as the input ids. For example, if
            users pass 3 instance_ids and the second instance could not be found,
            then returned list should also have 3 elements, with the 2nd elements being None.
        """
        id_batches = [
            instance_ids[i : i + AWS_API_INPUT_SIZE_LIMIT]
            for i in range(0, len(instance_ids), AWS_API_INPUT_SIZE_LIMIT)
        ]
        container_batches = [
            self.ecs_gateway.describe_tasks(self.cluster, ids) for ids in id_batches
        ]
        return sum(container_batches, [])

    def cancel_instance(self, instance_id: str) -> None:
        return self.ecs_gateway.stop_task(cluster=self.cluster, task_id=instance_id)

    def cancel_instances(self, instance_ids: List[str]) -> List[Optional[PcpError]]:
        res = []
        for instance_id in instance_ids:
            try:
                res.append(self.cancel_instance(instance_id))
            except PcpError as err:
                res.append(err)

        return res

    def get_current_instances_count(self) -> int:
        cluster = self.ecs_gateway.describe_cluster(self.cluster)
        return cluster.running_tasks + cluster.pending_tasks

    def get_cluster_instance(self) -> Cluster:
        cluster = self.ecs_gateway.describe_cluster(self.cluster)
        return cluster



#!/usr/bin/env python3
# Copyright (c) Meta Platforms, Inc. and affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

from typing import Any, Dict, List, Optional

from fbpcp.decorator.error_handler import error_handler
from fbpcp.gateway.gcp import GCPGateway
from google.cloud import storage
from google.oauth2.service_account import Credentials


class GCSGateway(GCPGateway):
    def __init__(
        self,
        credentials_json: Optional[str] = None,
        config: Optional[Dict[str, Any]] = None,
    ) -> None:
        super().__init__(credentials_json=credentials_json, config=config)
        credentials = (
            (Credentials.from_service_account_info(self.config["credentials_json"]))
            if "credentials_json" in self.config.keys()
            else None
        )
        project = credentials.project_id if credentials is not None else None

        self.client = storage.Client(
            project=project,
            credentials=credentials,
        )

    @error_handler
    def create_bucket(self, bucket: str, location: Optional[str] = None) -> None:
        # If location is none, then the bucket will be created in US regions (multi-region)
        self.client.create_bucket(bucket, location=location)

    @error_handler
    def delete_bucket(self, bucket: str) -> None:
        bucket = self.client.get_bucket(bucket)
        bucket.delete()

    @error_handler
    def upload_file(self, file_name: str, bucket: str, key: str) -> None:
        # TODO: Add ProgressBar
        bucket = self.client.bucket(bucket)
        blob = bucket.blob(key)
        blob.upload_from_filename(file_name)

    @error_handler
    def download_file(self, bucket: str, key: str, file_name: str) -> None:
        # TODO: Add ProgressBar
        bucket = self.client.bucket(bucket)
        blob = bucket.blob(key)
        blob.download_to_filename(file_name)

    @error_handler
    def put_object(self, bucket: str, key: str, data: str) -> None:
        bucket = self.client.bucket(bucket)
        blob = bucket.blob(key)
        blob.upload_from_string(data)

    @error_handler
    def get_object(self, bucket: str, key: str) -> str:
        bucket = self.client.bucket(bucket)
        blob = bucket.blob(key)
        return blob.download_as_string()

    @error_handler
    def get_object_size(self, bucket: str, key: str) -> int:
        bucket = self.client.bucket(bucket)
        blob = bucket.get_blob(key)
        return blob.size

    @error_handler
    def get_object_info(self, bucket: str, key: str) -> Dict[str, Any]:
        bucket = self.client.bucket(bucket)
        blob = bucket.get_blob(key)
        return {"size": blob.size, "updated": blob.updated}

    @error_handler
    def list_objects(self, bucket: str, key: str) -> List[str]:
        """
        Bucket:
            fileA
            fileB
            different_prefix_file
            folderA/fileC
            folderB/fileD

        list_blobs(bucker, prefix='file', delimiter='/') ->
            fileA
            fileB

        list_blobs(bucker, prefix='folderA/', delimiter='/') ->
            folderA/
            folderA/fileC
        """
        blobs = self.client.list_blobs(bucket, prefix=key)
        objects = []
        for blob in blobs:
            objects.append(blob.name)
        return objects

    @error_handler
    def delete_object(self, bucket: str, key: str) -> None:
        bucket = self.client.bucket(bucket)
        blob = bucket.blob(key)
        blob.delete()

    @error_handler
    def object_exists(self, bucket: str, key: str) -> bool:
        bucket = self.client.bucket(bucket)
        blob = bucket.blob(key)
        return blob.exists()

    @error_handler
    def copy(
        self,
        source_bucket_name: str,
        source_key: str,
        dest_bucket_name: str,
        dest_key: str,
    ) -> None:
        source_bucket = self.client.bucket(source_bucket_name)
        source_blob = source_bucket.blob(source_key)
        dest_bucket = self.client.bucket(dest_bucket_name)

        source_bucket.copy_blob(source_blob, dest_bucket, dest_key)



# Copyright (c) Meta Platforms, Inc. and affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

from re import sub
from typing import Any, Dict, Optional

import boto3
from fbpcp.gateway.aws import AWSGateway
from pce.entity.iam_role import IAMRole, RoleId, RoleName
from pce.mapper.aws import map_attachedrolepolicies_to_rolepolicies


class IAMGateway(AWSGateway):
    def __init__(
        self,
        region: str,
        access_key_id: Optional[str] = None,
        access_key_data: Optional[str] = None,
        config: Optional[Dict[str, Any]] = None,
    ) -> None:
        super().__init__(region, access_key_id, access_key_data, config)

        # pyre-ignore
        self.client = boto3.client("iam", region_name=self.region, **self.config)

    @classmethod
    def _role_id_to_name(cls, role_id: RoleId) -> RoleName:
        return sub(r".*?:role/", "", role_id)

    def get_policies_for_role(
        self,
        role_id: RoleId,
    ) -> Optional[IAMRole]:
        return map_attachedrolepolicies_to_rolepolicies(
            role_id,
            {
                policy["PolicyName"]:
                # Retrieving the policy document requires retrieving the version id for each policy arn, hence `get_policy` has to be called for each `get_policy_version` invocation
                self.client.get_policy_version(
                    PolicyArn=policy["PolicyArn"],
                    VersionId=self.client.get_policy(PolicyArn=policy["PolicyArn"])[
                        "Policy"
                    ]["DefaultVersionId"],
                )["PolicyVersion"]["Document"]
                for policy_result in self.client.get_paginator(
                    "list_attached_role_policies"
                ).paginate(RoleName=self._role_id_to_name(role_id))
                for policy in policy_result["AttachedPolicies"]
            },
        )



#!/usr/bin/env python3
# Copyright (c) Meta Platforms, Inc. and affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

from typing import Any, Dict, Optional

from pce.entity.iam_role import IAMRole, PolicyContents, PolicyName, RoleId


def map_attachedrolepolicies_to_rolepolicies(
    role_id: RoleId,
    attached_role_policies: Dict[PolicyName, PolicyContents],
) -> Optional[IAMRole]:
    return IAMRole(role_id, attached_role_policies) if attached_role_policies else None


def map_ecstaskdefinition_to_awslogsgroupname(
    task_definition: Dict[str, Any],
) -> Optional[str]:
    try:
        container_definition = task_definition["containerDefinitions"][0]
        log_configuration = container_definition["logConfiguration"]
        log_group_name = log_configuration["options"]["awslogs-group"]
        return log_group_name
    # Key error indicates the log group does not exist, error will be returned in error message templates
    except KeyError:
        return None



#!/usr/bin/env python3
# Copyright (c) Meta Platforms, Inc. and affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

from typing import Optional

from cryptography import x509
from cryptography.x509.oid import NameOID
from fbpcp.entity.certificate_request import CertificateRequest


def map_certificaterequest_to_x509name(cert_request: CertificateRequest) -> x509.Name:
    name_list = []
    if cert_request.country_name:
        name_list.append(
            x509.NameAttribute(NameOID.COUNTRY_NAME, cert_request.country_name)
        )
    if cert_request.state_or_province_name:
        name_list.append(
            x509.NameAttribute(
                NameOID.STATE_OR_PROVINCE_NAME, cert_request.state_or_province_name
            )
        )
    if cert_request.locality_name:
        name_list.append(
            x509.NameAttribute(NameOID.LOCALITY_NAME, cert_request.locality_name)
        )
    if cert_request.organization_name:
        name_list.append(
            x509.NameAttribute(
                NameOID.ORGANIZATION_NAME, cert_request.organization_name
            )
        )
    if cert_request.common_name:
        name_list.append(
            x509.NameAttribute(NameOID.COMMON_NAME, cert_request.common_name)
        )

    if not name_list:
        raise Exception(
            "x509 name is empty, please add at least one x509 name attribute in CertificateRequest"
        )
    return x509.Name(name_list)


def map_certificaterequest_to_x509subjectalternativename(
    cert_request: CertificateRequest,
) -> Optional[x509.SubjectAlternativeName]:
    name_list = []
    if cert_request.dns_name:
        name_list.append(x509.DNSName(cert_request.dns_name))
    return x509.SubjectAlternativeName(name_list)



#!/usr/bin/env python3
# Copyright (c) Meta Platforms, Inc. and affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.


"""
CLI for uploading an executable to one docker repo


Usage:
    onedocker-cli upload --config=<config> --package_name=<package_name> --package_path=<package_path> --version=<version>
    onedocker-cli archive --config=<config> --package_name=<package_name> [--version=<version> ] [options]
    onedocker-cli test --config=<config> --package_name=<package_name> --cmd_args=<cmd_args> [--version=<version> --timeout=<timeout>][options]
    onedocker-cli show --config=<config> --package_name=<package_name> [--version=<version>] [options]
    onedocker-cli stop --config=<config> --container=<container_id> [options]

Options:
    -h --help                Show this help
    --log_path=<path>        Override the default path where logs are saved
    --verbose                Set logging level to DEBUG
"""

import asyncio
import logging
import os
import time
from pathlib import Path, PurePath
from typing import Any, Dict, Optional

import schema
from docopt import docopt
from fbpcp.entity.container_instance import ContainerInstanceStatus
from fbpcp.service.container import ContainerService
from fbpcp.service.log import LogService
from fbpcp.service.onedocker import OneDockerService
from fbpcp.service.storage import StorageService
from fbpcp.util import reflect, yaml
from onedocker.repository.onedocker_repository_service import OneDockerRepositoryService

logger = None
onedocker_svc = None
container_svc = None
onedocker_repo_svc = None
log_svc = None
task_definition = None
repository_path = None
storage_svc = None

DEFAULT_TIMEOUT = 18000


def _upload(
    package_path: str,
    package_name: str,
    version: str,
) -> None:
    logger.info(
        f" Starting uploading package {package_name} at '{package_path}', version {version}..."
    )
    logger.info(f"Uploading binary for package {package_name}: {version}")
    onedocker_repo_svc.upload(package_name, version, package_path)
    logger.info(f" Finished uploading '{package_name}, version {version}'.\n")


def _archive(
    package_name: str,
    version: str,
) -> None:
    logger.info(f" Starting archiving package {package_name} version {version}...")
    logger.info(f"Archiving binary for package {package_name}: {version}")
    onedocker_repo_svc.archive_package(package_name, version)
    logger.info(f" Finished Archiving '{package_name}, version {version}'.\n")


def _test(
    package_name: str,
    version: str,
    cmd_args: str,
    timeout: int,
) -> None:
    logger.info(" Running test container ...")
    container = onedocker_svc.start_container(
        package_name=package_name,
        version=version,
        cmd_args=cmd_args,
        timeout=timeout,
    )
    container = asyncio.run(
        onedocker_svc.wait_for_pending_container(container.instance_id)
    )
    logger.info(container)
    log_path = log_svc.get_log_path(container)
    start_time = 0
    while container.status == ContainerInstanceStatus.STARTED:
        container = container_svc.get_instance(container.instance_id)
        if not container:
            raise ValueError(
                f"An error was raised while getting a container with id: {container.instance_id}, got None."
            )

        log_events = log_svc.fetch(log_path, start_time)

        for event in log_events:
            logger.info(event.message)

        if log_events:
            start_time = log_events[-1].timestamp + 1
        # temparorily set the process to be periodlly fetching and showing the log
        time.sleep(5)


def _show(
    package_name: str,
    version: Optional[str] = None,
) -> None:
    logger.info(
        f"Show package [{package_name}], version {version} information on storage "
    )

    if version:
        package_info = onedocker_repo_svc.package_repo.get_package_info(
            package_name, version
        )
        print(
            f" Package [{package_info.package_name}], version {package_info.version}: Last modified: {package_info.last_modified}; Size: {package_info.package_size} bytes"
        )
    else:
        package_versions = onedocker_repo_svc.package_repo.get_package_versions(
            package_name
        )
        print(
            f" All available versions for package {package_name} : {package_versions}"
        )
        for version in package_versions:
            package_info = onedocker_repo_svc.package_repo.get_package_info(
                package_name, version
            )
            print(
                f"Package [{package_info.package_name}], version {package_info.version}: Last modified: {package_info.last_modified}; Size: {package_info.package_size} bytes"
            )


def _stop(container_id: str) -> None:
    logger.info(f"Stopping container {container_id} ...")
    errors = onedocker_svc.stop_containers([container_id])
    if errors[0] is None:
        logger.info(f"Container {container_id} has been stopped")
    else:
        logger.info(
            f"Encountered errors when stopping container {container_id}: {errors[0]}"
        )


def _build_storage_service(config: Dict[str, Any]) -> StorageService:
    config_dependency: Optional[Dict[str, Any]] = config.get("dependency")
    if not config_dependency or "StorageService" not in config_dependency:
        raise KeyError("StorageService is absent in the config.")
    storage_svc_config: Dict[str, Any] = config_dependency["StorageService"]
    storage_class = reflect.get_class(storage_svc_config["class"])
    return storage_class(**storage_svc_config["constructor"])


def _build_container_service(config: Dict[str, Any]) -> ContainerService:
    config_dependency: Optional[Dict[str, Any]] = config.get("dependency")
    if not config_dependency or "ContainerService" not in config_dependency:
        raise KeyError("ContainerService is absent in the config.")
    container_svc_config: Dict[str, Any] = config_dependency["ContainerService"]
    container_class = reflect.get_class(container_svc_config["class"])
    return container_class(**container_svc_config["constructor"])


def _build_log_service(config: Dict[str, Any]) -> LogService:
    config_dependency: Optional[Dict[str, Any]] = config.get("dependency")
    if not config_dependency or "LogService" not in config_dependency:
        raise KeyError("LogService is absent in the config.")
    log_svc_config: Dict[str, Any] = config_dependency["LogService"]
    log_class = reflect.get_class(log_svc_config["class"])
    return log_class(**log_svc_config["constructor"])


def _build_exe_s3_path(repository_path: str, package_name: str, version: str) -> str:
    return f"{repository_path}{package_name}/{version}/{package_name.split('/')[-1]}"


def _build_repo_service(config: Dict[str, Any]) -> OneDockerRepositoryService:
    config_setting: Optional[Dict[str, str]] = config.get("setting")
    if not config_setting or "repository_path" not in config_setting:
        raise KeyError("repository_path is absent in the config.")
    storage_svc = _build_storage_service(config)
    return OneDockerRepositoryService(storage_svc, config_setting["repository_path"])


def _build_onedocker_service(config: Dict[str, Any], container_svc) -> OneDockerService:
    config_setting: Optional[Dict[str, str]] = config.get("setting")
    if not config_setting or "task_definition" not in config_setting:
        raise KeyError("task_definition is absent in the config.")
    return OneDockerService(container_svc, config_setting["task_definition"])


def main() -> None:
    global container_svc, onedocker_svc, onedocker_repo_svc, log_svc, logger, task_definition, repository_path, storage_svc
    s = schema.Schema(
        {
            "upload": bool,
            "test": bool,
            "show": bool,
            "stop": bool,
            "archive": bool,
            "--verbose": bool,
            "--help": bool,
            "--config": schema.And(schema.Use(PurePath), os.path.exists),
            "--package_name": schema.Or(None, schema.And(str, len)),
            "--package_path": schema.Or(None, schema.And(str, len)),
            "--cmd_args": schema.Or(None, schema.And(str, len)),
            "--container": schema.Or(None, schema.And(str, len)),
            "--log_path": schema.Or(None, schema.Use(Path)),
            "--version": schema.Or(None, schema.And(str, len)),
            "--timeout": schema.Or(None, schema.Use(int)),
        }
    )

    arguments = s.validate(docopt(__doc__))
    logger = logging.getLogger(__name__)
    log_path = arguments["--log_path"]
    log_level = logging.DEBUG if arguments["--verbose"] else logging.INFO
    logging.basicConfig(filename=log_path, level=log_level)

    package_name = arguments["--package_name"]
    package_path = arguments["--package_path"]
    version = arguments["--version"]

    config = yaml.load(Path(arguments["--config"])).get("onedocker-cli")

    if arguments["upload"]:
        onedocker_repo_svc = _build_repo_service(config)
        _upload(package_path, package_name, version)
    elif arguments["archive"]:
        onedocker_repo_svc = _build_repo_service(config)
        _archive(package_name, version)
    elif arguments["test"]:
        container_svc = _build_container_service(config)
        onedocker_svc = _build_onedocker_service(config, container_svc)
        log_svc = _build_log_service(config)
        timeout = arguments["--timeout"] if arguments["--timeout"] else DEFAULT_TIMEOUT
        _test(package_name, version, arguments["--cmd_args"], timeout)
    elif arguments["show"]:
        onedocker_repo_svc = _build_repo_service(config)
        _show(package_name, arguments["--version"])
    elif arguments["stop"]:
        container_svc = _build_container_service(config)
        onedocker_svc = _build_onedocker_service(config, container_svc)
        _stop(arguments["--container"])


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
# Copyright (c) Meta Platforms, Inc. and affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict
from datetime import datetime, timedelta
from typing import Optional, Union

from cryptography import x509
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives import hashes, serialization
from cryptography.hazmat.primitives.asymmetric import rsa
from cryptography.hazmat.primitives.serialization import (
    load_pem_private_key,
    load_pem_public_key,
)
from fbpcp.entity.certificate_request import KeyAlgorithm
from onedocker.entity.key_pair_details import KeyPairDetails


class CryptographyGateway:
    SIGN_ALGORITHM = hashes.SHA256()

    def _convert_str_to_bytes(self, s: str) -> bytes:
        return s.encode("utf-8")

    def generate_key_pair(
        self,
        key_algorithm: KeyAlgorithm,
        key_size: int,
        passphrase: Optional[str],
    ) -> KeyPairDetails:
        private_key = self._generate_private_key(key_algorithm, key_size)
        private_key_pem = self.get_private_key_pem(private_key, passphrase)
        public_key_pem = self.get_public_key_pem(private_key.public_key())
        return KeyPairDetails(
            private_key_pem=private_key_pem,
            public_key_pem=public_key_pem,
        )

    def _generate_private_key(
        self,
        key_algorithm: KeyAlgorithm,
        key_size: int,
    ) -> Union[rsa.RSAPrivateKeyWithSerialization]:
        if key_algorithm == KeyAlgorithm.RSA:
            # The public_exponent indicates what one mathematical property of the key generation will be. Official guide suggests always use 65537. (https://cryptography.io/en/latest/hazmat/primitives/asymmetric/rsa/)
            public_exponent = 65537
            private_key = rsa.generate_private_key(
                public_exponent=public_exponent,
                key_size=key_size,
            )
            return private_key
        else:
            # For now we only support RSA since it is one of the most common algorithms, will add more key algorithms in the future
            raise NotImplementedError

    def get_private_key_pem(
        self,
        private_key: Union[rsa.RSAPrivateKeyWithSerialization],
        passphrase: Optional[str],
    ) -> bytes:
        key_pem = private_key.private_bytes(
            encoding=serialization.Encoding.PEM,
            format=serialization.PrivateFormat.TraditionalOpenSSL,
            encryption_algorithm=serialization.BestAvailableEncryption(
                self._convert_str_to_bytes(passphrase)
            )
            if passphrase is not None
            else serialization.NoEncryption(),
        )
        return key_pem

    def load_private_key(
        self,
        key_pem: bytes,
        passphrase: Optional[str],
    ) -> Union[rsa.RSAPrivateKey]:
        private_key = load_pem_private_key(
            key_pem,
            self._convert_str_to_bytes(passphrase) if passphrase is not None else None,
            default_backend(),
        )
        return private_key

    def get_public_key_pem(
        self,
        public_key: Union[rsa.RSAPublicKey],
    ) -> bytes:
        key_pem = public_key.public_bytes(
            encoding=serialization.Encoding.PEM,
            format=serialization.PublicFormat.SubjectPublicKeyInfo,
        )
        return key_pem

    def load_public_key(self, key_pem: bytes) -> Union[rsa.RSAPublicKey]:
        public_key = load_pem_public_key(key_pem, default_backend())
        return public_key

    def _generate_certificate(
        self,
        subject_name: x509.Name,
        issuer_name: x509.Name,
        private_key: Union[rsa.RSAPrivateKey],
        public_key: Union[rsa.RSAPublicKey],
        not_valid_before: datetime,
        not_valid_after: datetime,
        subject_alternative_name: Optional[x509.SubjectAlternativeName] = None,
    ) -> x509.Certificate:
        builder = x509.CertificateBuilder(
            subject_name=subject_name,
            issuer_name=issuer_name,
            public_key=public_key,
            serial_number=x509.random_serial_number(),
            not_valid_before=not_valid_before,
            not_valid_after=not_valid_after,
        )
        if subject_alternative_name:
            builder.add_extension(
                subject_alternative_name,
                critical=False,
            )

        certificate = builder.sign(private_key, self.SIGN_ALGORITHM)
        return certificate

    def generate_certificate_pem(
        self,
        subject_name: x509.Name,
        issuer_name: x509.Name,
        key_pair_details: KeyPairDetails,
        passphrase: Optional[str],
        days_valid: int,
        subject_alternative_name: Optional[x509.SubjectAlternativeName] = None,
    ) -> bytes:
        not_valid_before = datetime.utcnow()
        not_valid_after = not_valid_before + timedelta(days=days_valid)

        public_key = self.load_public_key(key_pair_details.public_key_pem)
        private_key = self.load_private_key(
            key_pair_details.private_key_pem, passphrase
        )
        certificate = self._generate_certificate(
            subject_name=subject_name,
            issuer_name=issuer_name,
            private_key=private_key,
            public_key=public_key,
            not_valid_before=not_valid_before,
            not_valid_after=not_valid_after,
            subject_alternative_name=subject_alternative_name,
        )
        return self.get_certificate_pem(certificate)

    def get_certificate_pem(self, certificate: x509.Certificate) -> bytes:
        return certificate.public_bytes(serialization.Encoding.PEM)



# Copyright (c) Meta Platforms, Inc. and affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

from typing import Any, Dict, Optional

import boto3
import botocore
from fbpcp.gateway.aws import AWSGateway
from fbpcp.util.aws import split_container_definition
from pce.mapper.aws import map_ecstaskdefinition_to_awslogsgroupname


class ECSGateway(AWSGateway):
    def __init__(
        self,
        region: str,
        access_key_id: Optional[str] = None,
        access_key_data: Optional[str] = None,
        config: Optional[Dict[str, Any]] = None,
    ) -> None:
        super().__init__(region, access_key_id, access_key_data, config)

        self.client: botocore.client.BaseClient = boto3.client(
            "ecs", region_name=self.region, **self.config
        )

    def extract_log_group_name(self, container_definition_id: str) -> Optional[str]:
        task_definition_arn = split_container_definition(container_definition_id)[0]

        response = self.client.describe_task_definition(
            taskDefinition=task_definition_arn
        )
        log_group_name = map_ecstaskdefinition_to_awslogsgroupname(
            response["taskDefinition"]
        )
        return log_group_name



#!/usr/bin/env python3
# Copyright (c) Meta Platforms, Inc. and affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

import hashlib
from typing import Dict, List

from onedocker.entity.measurement import MeasurementType


class MeasurementService:
    def _get_content_bytes(self, file_path: str) -> bytes:
        with open(file_path, "rb") as file:
            content_bytes = file.read()
        return content_bytes

    def _generate_measurement(
        self,
        content_bytes: bytes,
        measurement_type: MeasurementType,
    ) -> str:
        hash_function = hashlib.new(measurement_type.value, content_bytes)
        return hash_function.hexdigest()

    def generate_measurements(
        self, measurement_types: List[MeasurementType], file_path: str
    ) -> Dict[MeasurementType, str]:
        content_bytes = self._get_content_bytes(file_path)
        measurements = {
            t: self._generate_measurement(content_bytes, t) for t in measurement_types
        }
        return measurements



# Copyright (c) Meta Platforms, Inc. and affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

from enum import Enum


class MPCRoles(Enum):
    PUBLISHER = "PUBLISHER"
    PARTNER = "PARTNER"
